#pragma TextEncoding = "Windows-1252"
#pragma rtGlobals=3		// Use modern global access method.
//#pragma version=1.17d// decided to not use pragma but a string constant 

//UMR Functions for High ToF-AMS analysis Toolkit v1.16
// Peak Integration by Key Analysis (PIKA)
//Copyright (C) 2005, 2006, 2007,2008, 2009,2010,2011, 2012, 2013, 2014, 2015, 2016, 2017
// Pete DeCarlo, Allison Aiken, Jose Jimenez, Donna Sueper University of Colorado
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
///////////////////////////////////////////////////////////////////////////////
// CURRENT STATUS
// see HR read me file for details.

//^ Functions liberally stolen from Allison & Pete
//^ Functions in this ipf deal with the calculations and grouping of exact masses species into families.
//^ The important thing to note is that the exact mass values as displayed in the stick_gr from one run
//^ are simply the peak heights from the fit.  When dealing with groups of species, it is important
//^ to incorporate the duty cycle correction and possibly the area conversion into the calculations.
//
// The UM prefix stands for Unit Mass

STRCONSTANT ION_FAMILY_LIST = "Cx;CH;CHO1;CHOgt1;CHN;CHO1N;CHOgt1N;CS;HO;NH;Cl;NO;SO;Air;Other;CSi"		// CS added in 1.04P
// These are more appropriately named CxHy, etc. Cl fragment should really be named  ClxAyBz.... 
// IonFamilyList should also match the text wave root:HR:HR_familyName.  Later versions may allow more flexibility with family or other grouping definitions


// Given the results from HR fitting of one spectra for open, closed and diff, sum individual ions (CH3O2 etc) peak heights into families (CxHyOz).
// Basically we go from individual ions to families
Function UM_DoAllCalcsForIntMassSpectrum()

	UM_GenerateAxByMaskWaves(root:HR_sticks:ExactMassText, "")
	UM_MakeIntResolutionSticks(root:HR_sticks:ExactMassWave, root:HR_sticks:OneRunStickDiff, "Diff", "root:Int_Sticks:int", "root:Int_Sticks:OneRunStickSumSpec" )
	UM_MakeIntResolutionSticks(root:HR_sticks:ExactMassWave, root:HR_sticks:OneRunStickClosed, "Closed", "root:Int_Sticks:int", "root:Int_Sticks:OneRunStickSumSpec")
	UM_MakeIntResolutionSticks(root:HR_sticks:ExactMassWave, root:HR_sticks:OneRunStickOpen, "Open", "root:Int_Sticks:int", "root:Int_Sticks:OneRunStickSumSpec")

	UM_DoAllCalcsForIntMSOMinusC()

End

Function UM_GenerateAxByMaskWaves(ExactMassText, maskWaveSuffix) //maskWaveSuffix is either an empty string or a _todoFit
	wave/t ExactMassText
	string maskWaveSuffix
	
	wave/t HR_familyName = root:HR:HR_familyName
	wave/z/t HR_familyListDefinition = root:HR:HR_familyListDefinition
	if (!WaveExists(HR_familyListDefinition))
		make/t/n=(numpnts(HR_familyName)) root:HR:HR_familyListDefinition = "default"
		wave/t HR_familyListDefinition = root:HR:HR_familyListDefinition	
	endif

	variable numC, numH , numO, numN, numS, numAr, numCl, numW, numSi		// for now we don't use W
	variable idex, kdex, numFamily, numExactMasses, exceptionWaveNum, exceptionFlag
	string fragment, exceptionStr, str1, str2
	string exceptionWaveList =""
	string exceptionMassList =""
	string exceptionFamilyList =""
	variable tempNum, totalNumExceptions=0
	
	numExactMasses = numpnts(ExactMassText)
	numFamily = numpnts(HR_familyName)
	
	NewDataFolder/o/s root:Int_Sticks	
	for (idex=0;idex<numpnts(HR_familyName);idex+=1)
		make/O/N=(numExactMasses) $"mask_"+HR_familyName[idex]+maskWaveSuffix = 0	//  must be zero and not nan
	endfor
	SetDataFolder root:
	
	pk_FindHRionExceptions(exceptionWaveList, exceptionMassList, exceptionFamilyList)

	for (idex = 0; idex < numExactMasses; idex += 1)
		
		Fragment = ExactMassText[idex]										
		
 		str1 = pk_FindFamilyFroHRIon(Fragment, exceptionMassList, exceptionFamilyList, HR_familyName)		//1.14E
		wave/z ThisFamily = $"root:Int_Sticks:mask_"+str1+maskWaveSuffix
		if (!WaveExists(ThisFamily))
			Abort "The family "+str1+" doesn't seem to exist.  Aborting from UM_GenerateAxByMaskWaves"
		endif
		ThisFamily[idex]=1	
			
	endfor
	
	make/o/t/n=(numExactMasses)$"root:HR_sticks:ExactMassFamilyText"+maskWaveSuffix/wave= ExactMassFamilyText
	
	for(idex=0; idex<numExactMasses; idex+=1)		// for each HR ion
		
		for (kdex = 0;kdex<numFamily;kdex+=1)	// go through all the families to find its match
			wave ThisFamily = $"root:Int_Sticks:mask_"+HR_familyName[kdex]+maskWaveSuffix			
			if (ThisFamily[idex] ==1)
				ExactMassFamilyText[idex] = HR_familyName[kdex]
				break
			endif
		endfor		
	endfor
	
	// generated in HR_findIsotopParent
	killwaves/z ParentElementCountWave, ParentHRnameElementOrder, ParentHRnameElementOrderIndex

End


// new in 1.14E
Function/s pk_FindFamilyFroHRIon(Fragment, exceptionMassList, exceptionFamilyList, HR_familyName)
string Fragment, exceptionMassList, exceptionFamilyList
wave/t HR_familyName

	string familyStr=""
	variable exceptionFlag, kdex, tempNum
	variable numC, numH , numO, numN, numS, numAr, numCl, numW, numSi, num	// for now we don't use W
	
	// first check to see if it is any of our execeptions
	exceptionFlag=0
	kdex = WhichListItem(fragment, exceptionMassList)
	if (kdex>=0)
		tempNum = str2num(stringFromList(kdex, exceptionFamilyList))
		familyStr = HR_familyName[tempNum]
		return familyStr
	endif
	
	Fragment = ReplaceString("inV", Fragment, "")	// not relevant for family categorization
	Fragment = ReplaceString("inW", Fragment, "")		// not relevant for family categorization
	Fragment = ReplaceString("plus2", Fragment, "")		// not relevant for family categorization
	if (strsearch(Fragment, "j", 0) >=0)
		Fragment = HR_FindIsotopParent(Fragment)		 // translate the isotopes to their parent for easier categorization, so Oj18O becomes O2
	endif
	 		
	numC = HR_ReturnNumElements(Fragment, "C", "a|u|l|r|o")
	numH = HR_ReturnNumElements(Fragment, "H", "g|f|e")
	numO = HR_ReturnNumElements(Fragment, "O", "x")		// we need one bad one, hence, 'x'
	numN = HR_ReturnNumElements(Fragment, "N", "e|a|i|b|d|o")
	numS = HR_ReturnNumElements(Fragment, "S", "e|a|i|b|d|o|n")
	numAr = HR_ReturnNumElements(Fragment, "Ar", "x")		// we need one bad one, hence, 'x'
	numCl = HR_ReturnNumElements(Fragment, "Cl", "x")		// we need one bad one, hence, 'x'
	numSi = HR_ReturnNumElements(Fragment, "Si", "x")		// we need one bad one, hence, 'x'
	numW = HR_ReturnNumElements(Fragment, "W", "x")		// we need one bad one, hence, 'x'  should have removed all inW text

	make/o/free/t KnownAtoms = {"C","H","O","N","S","Ar","Cl","Si", "W"}
	make/o/free/n=(numpnts(KnownAtoms)) KnownAbund // = {numC,numH,numO,numN,numS,numAr,numCl,numSi,numW}
	
	if (numC>0)		//C>0
		if (numSi>0)	//C>0,S>0	// added in 1.10Q
			familyStr="CSi"
		elseif (numS>0)	//C>0,S>0
			familyStr="CS"
		elseif(numN>0)	// C>0, S=0, N>0	
			if (numO>=2)  // C>0, S=0, N>0, O>=2
				familyStr="CHOgt1N"
			elseif(numO==1)  // C>0, S=0, N>0, O=1
				familyStr="CHO1N"
			else		//  C>0, S=0, N>0, O=0
				familyStr="CHN"
			endif			
		elseif(numN==0)		// C>0, S=0, N==0 
			if (numO>=2)  // C>0, S=0, N=0, O>=2
				familyStr="CHOgt1"
			elseif(numO==1)	// C>0, S=0, N=0, O=1
				familyStr="CHO1"
			else		// C>0, S=0, N=0, O=0
				if (numH>0) // C>0, S=0, N=0, O=0, H>0
					familyStr="CH"
				elseif( stringmatch(Fragment, "C") || str2num(stringFromList(1, Fragment, "C"))>0) // C>0 there is only carbon
					familyStr="Cx"
				else // C>0, S=0, N=0, O=0, H=0 and there are other elements besides carbon
					familyStr="Other"
				endif
			endif			
		endif
	elseif(numN>0)	// C=0, N>0 		
		if (numO>0)	// C=0, N>0, O>0
			familyStr="NO"
		else		// C=0, N>0, O=0
			if (numH>0)	// C=0, N>0, O=0, H>0
				familyStr="NH"
			elseif( stringmatch(Fragment, "N")  ||  stringmatch(Fragment, "N2")  )		//N2 or N 
				familyStr="air"
			else		// C=0, N>0, O=0, H=0, not N2 or N, must be a strange beast
				familyStr="other"
			endif
		endif		
	elseif(numS>0)	//C=0,N=0,S>0	// note that we took care of CS ions above 		
		if (numO>0 || stringmatch(fragment, "S"))
			familyStr="SO"
		else		// C=0, S>0, O==0, must be a strange beast
			familyStr="other"
		endif		
	elseif(numO>0) 		// C=0, N=0, S=0, O>0
		if (numH>0)		// C=0, N=0, S=0, O>0,H>O
			 KnownAbund = {numC,numH,numO,numN,numS,numAr,numCl,numSi,numW}
			 if (strlen(pk_RemainderFormula(Fragment, KnownAtoms, KnownAbund))>0)
				familyStr="Other"
			else
				familyStr="HO"
			endif
		elseif (stringmatch(Fragment, "O")  ||  stringmatch(Fragment, "O2") || stringmatch(Fragment, "O3"))	
			familyStr="air"
		else			// C=0, N=0, S=0, O>0,H=O, not O, O2 or O3,  must be a strange beast
			familyStr="other"
		endif		
	elseif(numAr>0) 		// C=0, N=0, S=0, O=0, Ar>0
		familyStr="air"
	elseif(numCl>0) 		// C=0, N=0, S=0, O=0, Ar=0, Cl>0
		num =HR_OneHRIonElementalComp(Fragment)  //1.21B
		if ((num==2 && numH==1) ||(num==1 && numH==0) )
			familyStr="Cl"			//1.21B
		else
			familyStr="Other"
		endif
	elseif (numW>0)	//C>0,S>0	// added in 1.10Q
		familyStr="Tungsten"
	else
		// C=0, N=0, S=0, O=0, Ar=0, Cl=0 , W=0
		familyStr="other"
	endif	
	
	if (strlen(familyStr) == 0)
		abort "Could not find a family for HR ion " + fragment+"  Aborting from pk_FindFamilyFroHRIon"
	endif
	
	return familyStr
			
End


//1.04P addition
Function UM_make3DColorWave()

	wave/t ExactMassText = root:HR_Sticks:ExactMassText
	wave/t HR_familyName = root:HR:HR_familyName
	wave HR_family_R = root:HR:HR_family_R
	wave HR_family_G = root:HR:HR_family_G
	wave HR_family_B = root:HR:HR_family_B
	
	variable numExactMasses, idex, pos
	
	numExactMasses = numpnts(ExactMassText)

	make/i/o/n=(numExactMasses, 3) root:HR_Sticks:ExactMassColorTable
	wave ExactMassColorTable = root:HR_Sticks:ExactMassColorTable
	
	for(idex=0;idex<numExactMasses;idex+=1)
		pos = pk_FindWhichfamily(idex, HR_familyName)
		ExactMassColorTable[idex][0] = HR_family_R[pos]
		ExactMassColorTable[idex][1] = HR_family_G[pos]	
		ExactMassColorTable[idex][2] = HR_family_B[pos]	
	endfor

End


// 1.04P addition
Function PK_FindWhichfamily(pos, HR_familyName)
variable pos
wave/t HR_familyName

	string myWaveList, myWaveStr
	variable idex, num, result

	num = numpnts(HR_familyName)
	result = -1
	do
		wave mask = $"root:Int_sticks:mask_"+HR_familyName[idex]
		if (mask[pos]==1)
			result = idex
		endif
		idex+=1
	while(idex< num && result<0)
	
	return result
	
End


// Create the wave root:Int_Sticks:dutyCycleCorr and inserts appropriate duty cycle values.  Note that this is ExactMassWave dependent.
Function UM_MakeDutyCycleCorr()

	wave ExactMassWave = root:HR_Sticks:ExactMassWave
	nvar dutycycle_mz=root:panel:dutycycle_mz		// typically = 28

	make/O/N=(numpnts(ExactMassWave))  root:Int_Sticks:dutyCycleCorr
	wave dutyCycleCorr = root:Int_Sticks:dutyCycleCorr
	dutyCycleCorr = sqrt(dutycycle_mz /(ExactMassWave[p]))		// one wave

End


//^ Significantly overhauled from Pete & Allison's code
// done in 1.02 to reflect that we are passing real stick values (areas under curves) instead of peak heights 
// Should only be called for one spectra fit
Function UM_MakeIntResolutionSticks(ExactMassWave, ExactStickWave, spectraType, waveNamePrefix, sumWavePrefix)
wave ExactMassWave, ExactStickWave
string spectraType, waveNamePrefix, sumWavePrefix

	string FamilyStickStr	
	variable iExactMasses, numExactMasses, maxAmu, onePeakVal, peakShapeScalar, integerVal
	variable idex, num
	
	wave UnitMass = root:HR_Sticks:UnitMass
	
	wave/t/z HR_familyName = root:HR:HR_familyName		// 1.08A error found by Jurgita
	if (!WaveExists(HR_familyName))
		HR_MakeHRFamilyBatch()
		wave/t/z HR_familyName = root:HR:HR_familyName
	endif
	HR_makeHRFamilyLegend()

	num=numpnts(HR_familyName)	
	
	numExactMasses = numpnts(ExactMassWave)
	wavestats/q/m=1 ExactMassWave
	maxAmu = round(V_max)
			
	for(idex=0; idex<num; idex+=1)		//^ initialize all integer sticks to 0
		FamilyStickStr = HR_familyName[idex]
		make /O /N=(maxAmu) $waveNamePrefix+"_"+FamilyStickStr+"_"+SpectraType = 0
		Setscale x, 1,1,  $waveNamePrefix+"_"+FamilyStickStr+"_"+SpectraType		//1.04J
	endfor
	
	make /O /N=(maxAmu) $sumWavePrefix+SpectraType=0
	wave OneRunStickSumSpec =  $sumWavePrefix+SpectraType

	for(iExactMasses=0; iExactMasses <numExactMasses; iExactMasses +=1)		// go through each point, each mass in HR stick wave.  fillin entries in the intWave wave
	
		for(idex=0; idex<num; idex+=1)	// loop through all possible families
			FamilyStickStr = HR_familyName[idex]
			wave  MaskWave = $"root:Int_Sticks:mask_"+FamilyStickStr	//CH_Mask
			
			if(MaskWave[iExactMasses] == 1)
				integerVal=round(ExactMassWave[iExactMasses])
				wave intWave = $waveNamePrefix+"_"+FamilyStickStr+"_"+SpectraType
				onePeakVal = max(0,ExactStickWave[iExactMasses])	// need to check to see why sometimes we get neg peaks - Igor Pro issue?
				intWave[integerVal-1] += onePeakVal		// convert from perfect gaussian to custom peak shape
				break		// break out of for (i=0..) loop because if it is one family it shouldn't be in any other family
			endif					
		endfor
	endfor
	
	for(idex=0; idex<num; idex+=1)	// loop through all possible families
		FamilyStickStr = HR_familyName[idex]
		wave intWave = $waveNamePrefix+"_"+FamilyStickStr+"_"+SpectraType		
		OneRunStickSumSpec+=intWave
	endfor
		
End


// 1.08E
Function UM_DoAllCalcsForIntMSOMinusC()

	wave/t HR_familyName = root:HR:HR_familyName
	variable idex, num
	string FamilyStickStr
		
	num=numpnts(HR_familyName)
	
	for(idex=0; idex<num; idex+=1)		//^ initialize all integer sticks to 0
		FamilyStickStr = HR_familyName[idex]
		wave intWaveOpen = $"root:Int_Sticks:int_"+FamilyStickStr+"_Open"
		wave intWaveClosed = $"root:Int_Sticks:int_"+FamilyStickStr+"_Closed"
		make /O /N=(numpnts(intWaveClosed)) $"root:Int_Sticks:int_"+FamilyStickStr+"_OMinusC"
		wave OminusC = $"root:Int_Sticks:int_"+FamilyStickStr+"_OMinusC"
		Setscale x, 1,1,  OminusC		//1.04J
		OminusC = intWaveOpen - intWaveClosed
	endfor

End


// retrieves SQ stick values for comparison with Pika values
// waves are put in root:Int_Sticks:MSSClosed_p_OneSpectra, etc.
Function UM_ExtractSQMassSpectra()

	svar sq_todoWvNm = root:panel:sq_todoWvNm
 	wave todo = $sq_todoWvNm
 	nvar HR_checkDoNotGrabRawSpectra = root:HR_panel:HR_checkDoNotGrabRawSpectra  // misname should be single run or average todo
	nvar HR_runVar = root:HR_Panel:HR_runVar

	string paramStr

 	paramStr = "speclist:all/;fraglist:all/;rie_list:1/;ce_list:1/;spec_corr:;raw_flag:0;cache_op:0;filter_op:1;corr_op:0;product_type:2;ug_op:0;findBaseline_op:0;"

 	if (HR_checkDoNotGrabRawSpectra)	// working with an averaged HR spectra
//		if (stringmatch("OPEN", upperStr(spectraType))	)		// we have to do this differently for open because MSSOpen_p doesn't exist
			squirrel_fetch(todo,sq_MS_concentration,ParamStr+"destlist:MSSC_"+sq_todoWvNm+"_temp;","MSSClosed_p")
			squirrel_fetch(todo,sq_MS_concentration,ParamStr+"destlist:MSSD_"+sq_todoWvNm+"_temp;","MSSDiff_p")
			wave MSSC_temp = $"MSSC_"+sq_todoWvNm+"_temp"
			wave MSSD_temp = $"MSSD_"+sq_todoWvNm+"_temp"
			duplicate/o MSSC_temp root:Int_Sticks:MSSClosed_p_OneSpectra
			duplicate/o MSSD_temp root:Int_Sticks:MSSDiff_p_OneSpectra
			duplicate/o MSSC_temp root:Int_Sticks:MSSOpen_p_OneSpectra
			wave MSSClosed_p_OneSpectra = root:Int_Sticks:MSSOpen_p_OneSpectra
			wave MSSDiff_p_OneSpectra = root:Int_Sticks:MSSOpen_p_OneSpectra
			wave MSSOpen_p_OneSpectra = root:Int_Sticks:MSSOpen_p_OneSpectra
// 			MSSOpen_p_OneSpectra = MSSClosed_p_OneSpectra + MSSDiff_p_OneSpectra
 			killwaves/z MSSC_temp, MSSD_temp
// 		else
//			squirrel_fetch(todo,sq_MS_concentration,ParamStr+"destlist:MSS_"+sq_todoWvNm+"_temp;","MSS"+spectraType+"_p")
//			wave MSS_temp = $"MSS_"+sq_todoWvNm+"_temp"
//			duplicate/o MSS_temp $"root:Int_Sticks:MSS"+spectraType+"_p_OneSpectra"
// 			killwaves/z MSS_temp
// 		endif
	else		// 		working with a single run
//		if (stringmatch("OPEN", upperStr(spectraType))	)// we have to do this differently for open because MSSOpen_p doesn't exist
			sq_GetThisRawMS(HR_runVar, "Int_Sticks", "MSSClosed_p_OneSpectra", "MSSClosed_p")
			sq_GetThisRawMS(HR_runVar, "Int_Sticks", "MSSDiff_p_OneSpectra", "MSSDiff_p")
			wave MSSClosed_p_OneSpectra = root:Int_Sticks:MSSClosed_p_OneSpectra
			wave MSSDiff_p_OneSpectra      = root:Int_Sticks:MSSDiff_p_OneSpectra
			duplicate/o MSSDiff_p_OneSpectra root:Int_Sticks:MSSOpen_p_OneSpectra
			wave MSSOpen_p_OneSpectra = root:Int_Sticks:MSSOpen_p_OneSpectra
//			MSSOpen_p_OneSpectra = MSSClosed_p_OneSpectra+MSSDiff_p_OneSpectra
//		else
//			sq_GetThisRawMS(HR_runVar, "Int_Sticks", "MSS"+spectraType+"_p_OneSpectra", "MSS"+spectraType+"_p")		
//		endif
	endif

 	variable minmz = min (numpnts(MSSClosed_p_OneSpectra), numpnts(MSSDiff_p_OneSpectra) )		//1.59E
 	redimension/n=(minmz) MSSOpen_p_OneSpectra
 	
 	MSSOpen_p_OneSpectra = MSSClosed_p_OneSpectra + MSSDiff_p_OneSpectra
End

//^ Makes a integer spectra plot of where ions have been grouped into families and summed ('Pete's graph 6').  
// It doesn't plot the air or other species.
Function UM_PlotIntMassSpectra(spectraType, SingleGraph)
string spectraType
variable SingleGraph

	wave squirrel_index = root:index:squirrel_index
	svar HR_legendStrUnitSticks = root:Int_Sticks:HR_legendStrUnitSticks
	nvar HR_runVar = root:HR_Panel:HR_runVar
	
	svar sq_todoWvNm = root:panel:sq_todoWvNm
 	wave todo = $sq_todoWvNm
 	nvar HR_checkDoNotGrabRawSpectra = root:HR_panel:HR_checkDoNotGrabRawSpectra		// misnamed, should be single run or average todo

	variable idex, num
	string FamilyStickStr, legendStr, paramStr
	
	legendStr = "\\Z12"+spectraType+"\r"

	wave/t/z HR_familyName = root:HR:HR_familyName
	wave/z HR_family_R = root:HR:HR_family_R
	wave/z HR_family_G = root:HR:HR_family_G
	wave/z HR_family_B = root:HR:HR_family_B
	if (waveexists(HR_familyName)==0)		// assume that if one is missing, they all are.
		HR_MakeHRFamilyBatch()		// I think we always want to redo... 1.04P
		wave/t HR_familyName = root:HR:HR_familyName
		wave HR_family_R = root:HR:HR_family_R
		wave HR_family_G = root:HR:HR_family_G
		wave HR_family_B = root:HR:HR_family_B
	endif
	
	HR_makeHRFamilyLegend()		//103C
	
	// first check to see that the data type exists

	num = numpnts(HR_familyName)

	if (SingleGraph)
		DoWindow/F $"UnitMassSpectra_"+SpectraType
		if (V_flag==0)
			Display /W=(150,100,550,350)/N=$("UnitMassSpectra_"+SpectraType) as "UnitMassSpectra_"+SpectraType
		else
			return 1
		endif
	endif
	
	wave MSSWave = $"root:Int_Sticks:MSS"+spectraType+"_p_OneSpectra"	
//	AppendToGraph/B=UnitStickBottom/L=$"UnitStick"+spectraType MSSWave  
	AppendToGraph MSSWave  

	for  (idex =0;idex<num;idex+=1)
		FamilyStickStr = HR_familyName[idex]
		wave/z int_wave = $"root:Int_Sticks:int_"+FamilyStickStr+"_"+spectraType
		if(waveExists(int_wave)) 
	//		AppendToGraph/B=UnitStickBottom/L=$"UnitStick"+spectraType int_wave 		//1.09H 
			AppendToGraph int_wave  
			ModifyGraph  rgb($NameOfWave(int_wave))=(HR_family_R[idex], HR_family_G[idex], HR_family_B[idex] )
			ModifyGraph mode($NameOfWave(int_wave))=1,toMode($NameOfWave(int_wave))=2
			ModifyGraph offset($NameOfWave(int_wave))={1,0}
			ModifyGraph lsize($NameOfWave(int_wave))=2
		endif
	endfor
		
	if (SingleGraph)
		ModifyGraph margin(left)=54,margin(bottom)=54,margin(top)=18,margin(right)=18
		ModifyGraph mode=1
		ModifyGraph lSize=2
		ModifyGraph toMode=3
		ModifyGraph zero($"UnitStick"+spectraType)=1
		ModifyGraph nticks(UnitStickBottom)=10
		ModifyGraph minor(UnitStickBottom)=1
		ModifyGraph fSize=10
		ModifyGraph freePos($"UnitStick"+spectraType)=0,freePos(UnitStickBottom)=0
		ModifyGraph lblMargin($"UnitStick"+spectraType)=5
		ModifyGraph lblMargin($"UnitStick"+spectraType)=5
		ModifyGraph lblPos($"UnitStick"+spectraType)=52,lblPos(UnitStickBottom)=48
		ModifyGraph lblLatPos($"UnitStick"+spectraType)=4,lblLatPos(UnitStickBottom)=-12
		Label $"UnitStick"+spectraType "\\Z16Signal (Arb. Units)"
		Label UnitStickBottom "(m/z)"
		SetAxis/A/E=1 $"UnitStick"+spectraType
		SetAxis UnitStickBottom 10,100
		TextBox/N=text0/J/X=1.25/Y=-4.42 legendStr+HR_legendStrUnitSticks
	endif

	ModifyGraph mode($NameOfWave(MSSWave))=3,msize($NameOfWave(MSSWave))=4, mrkThick($NameOfWave(MSSWave))=1
	ModifyGraph rgb($NameOfWave(MSSWave))=(0,0,0)
	ModifyGraph offset($NameOfWave(MSSWave))={1,0}
//	ReorderTraces $"int_CH_"+spectraType ,{$"MSS"+spectraType+"_p_OneSpectra"}

End
// Creates root:HR:HR_familyName and RGB color waves and string legend HR_legendStrUnitSticks used for plotting.
// In future versions it may be possible for users to define their own family and its associated colors.  (the string constant 
// in 1.02A the addition of the tungsten family is mostly for the purpose of demonstration, it may not have too much scientific value
Function HR_MakeHRFamilyBatch()

//	if (Exists("root:HR:HR_familyName"))		// 1.03A
//		return 0
//	endif
	
	variable idex

	make/t/o/n=16 root:HR:HR_familyName, root:HR:HR_familyListDefinition
	make/o/n=16 root:HR:HR_family_R, root:HR:HR_family_G, root:HR:HR_family_B,  root:HR:HR_family_pointNumber
	make/o/n=(16,3) root:HR:HR_family_2dColorTable	// 1.04P
	wave/t HR_familyName = root:HR:HR_familyName
	wave/t HR_familyListDefinition = root:HR:HR_familyListDefinition		//1.02A
	wave HR_family_R = root:HR:HR_family_R
	wave HR_family_G = root:HR:HR_family_G
	wave HR_family_B = root:HR:HR_family_B
	wave HR_family_pointNumber = root:HR:HR_family_pointNumber
	wave HR_family_2dColorTable = root:HR:HR_family_2dColorTable

	HR_family_pointNumber = p+1
	
	idex=0
	HR_familyName[idex] = "Cx"; 		HR_family_R[idex] =0; 		HR_family_G[idex] =0; 		HR_family_B[idex] =0; 		idex+=1
	HR_familyName[idex] = "CH"; 		HR_family_R[idex] =0; 		HR_family_G[idex] =39168; 	HR_family_B[idex] =0; 		idex+=1
	HR_familyName[idex] = "CHO1"; 	HR_family_R[idex] =32640; 	HR_family_G[idex] =0; 		HR_family_B[idex] =29440; 	idex+=1
	HR_familyName[idex] = "CHOgt1";	HR_family_R[idex] =65280; 	HR_family_G[idex] =0; 		HR_family_B[idex] =58880; 	idex+=1
	HR_familyName[idex] = "CHN"; 	HR_family_R[idex] =36864; 	HR_family_G[idex] =14592; 	HR_family_B[idex] =58880; 	idex+=1
	HR_familyName[idex] = "CHO1N"; 	HR_family_R[idex] =8192; 		HR_family_G[idex] =24448; 	HR_family_B[idex] =32640; 	idex+=1
	HR_familyName[idex]= "CHOgt1N"; HR_family_R[idex] =16384; 	HR_family_G[idex] =48896; 	HR_family_B[idex] =65280; 	idex+=1
	HR_familyName[idex] = "CS"; 		HR_family_R[idex] =8192; 	HR_family_G[idex] =8192; 		HR_family_B[idex] =65280; 	idex+=1
	HR_familyName[idex] = "HO"; 		HR_family_R[idex] =0; 		HR_family_G[idex] =0; 		HR_family_B[idex] =0; 		idex+=1	
	HR_familyName[idex] = "NH"; 		HR_family_R[idex] =65280; 	HR_family_G[idex] =43520; 	HR_family_B[idex] =0; 		idex+=1
	HR_familyName[idex] = "Cl"; 		HR_family_R[idex] =65280; 	HR_family_G[idex] =0; 		HR_family_B[idex] =52224; 	idex+=1
	HR_familyName[idex] = "NO"; 		HR_family_R[idex] =0;	 	HR_family_G[idex] =65280; 	HR_family_B[idex] =65280; 	idex+=1
	HR_familyName[idex] = "SO"; 		HR_family_R[idex] =65280; 	HR_family_G[idex] =0; 		HR_family_B[idex] =0; 		idex+=1
	HR_familyName[idex] = "Air"; 		HR_family_R[idex] =36864; 	HR_family_G[idex] =36864; 	HR_family_B[idex] =36864; 	idex+=1
	HR_familyName[idex] = "Tungsten"; HR_family_R[idex] =0; 		HR_family_G[idex] =0; 		HR_family_B[idex] =0; 		idex+=1	//1.02A addition, default black color
	HR_familyName[idex] = "Other"; 	HR_family_R[idex] =16384; 	HR_family_G[idex] =16384; 	HR_family_B[idex] =39168; 	idex+=1

	HR_familyListDefinition = "default"
	idex = sq_findStr(HR_familyName, "Tungsten", 1)
	if (idex>=0)		//1.16Z
		HR_familyListDefinition[idex] = "j182Wplus2;j184Wplus2;j186Wplus2;W;WO"		// for purposed of demonstration for the tungsten family
	endif

	HR_family_2dColorTable[][0] = HR_family_R[p]
	HR_family_2dColorTable[][1] = HR_family_G[p]
	HR_family_2dColorTable[][2] = HR_family_B[p]

End

Function HR_makeHRFamilyLegend()

	wave/t HR_familyName = root:HR:HR_familyName
	wave HR_family_R = root:HR:HR_family_R
	wave HR_family_G = root:HR:HR_family_G
	wave HR_family_B = root:HR:HR_family_B

	variable idex, num

	num=numpnts(HR_familyName)
	string/g  root:Int_Sticks:HR_legendStrUnitSticks
	svar HR_legendStrUnitSticks = root:Int_Sticks:HR_legendStrUnitSticks
	
	HR_legendStrUnitSticks="Families:\r\f01"
	for (idex=0;idex<num;idex+=1)
		HR_legendStrUnitSticks+="\\K("+num2str(HR_family_R[idex])+","+num2str(HR_family_G[idex])+","+num2str(HR_family_B[idex])+")"+HR_familyName[idex]+"\r"
	endfor
	HR_legendStrUnitSticks=HR_legendStrUnitSticks[0, strlen(HR_legendStrUnitSticks)-2]

End


// Creates waves root:Int_sticks:FamilyMemWave which helps users see what ions are in what family.
Function HR_CreateFamilyMembers(maskWave)
wave maskWave

	wave ExactMassWave=root:HR_Sticks:ExactMassWave
	wave/t ExactMassText=root:HR_Sticks:ExactMassText
	
	variable idex, num, numFamMem=0

	num=numpnts(ExactMassText)
	make/o/n=(num) root:Int_sticks:FamilyMemWave
	make/t/o/n=(num) root:Int_sticks:FamilyMemText

	wave FamilyMemWave = root:Int_sticks:FamilyMemWave
	wave/t FamilyMemText = root:Int_sticks:FamilyMemText

	for(idex=0;idex<num;idex+=1)
		if (maskWave[idex]==1)	
			FamilyMemWave[numFamMem]=ExactMassWave[idex]
			FamilyMemText[numFamMem]=ExactMassText[idex]
			numFamMem+=1
		endif	
	endfor
	
	Deletepoints numFamMem, num, FamilyMemWave, FamilyMemText
	
End


/////////////////////////////////////////////////////////////////////////////////////////
//////////////      ISOTOPE FUNCTIONS          ///////////////// 
/////////////////////////////////////////////////////////////////////////////////////////

// This function creates a sublist of the chosen waves.  This sublist contains only isotopes.
// It checks that all parent isotopes have also been chosen to fit.
// It creates and fills in the following 11 waves in root:HR_Sticks:
// IsotopicMassWave
// IsotopicMassText
// IsotopicParentWave
// IsotopicParentText
// IsotopicAbund
// IsotopicMask
// IsotopicMassPt
// IsotopicParentPt
// IsotopicParentWaveRnd
// IsotopicChildWaveRnd
// IsotopicChildWaveRndRev
// some of which may be uneccessary if the user chooses to NOT constrain the isotopes.
// This is always called when masses are chosen.

Function HR_ExtractIsotopicValues()

	Wave ExactMassWave = root:HR_Sticks:ExactMassWave
	Wave/T ExactMassText = root:HR_Sticks:ExactMassText
	Wave/T ExactMassIsotopeRef = root:HR_Sticks:ExactMassIsotopeRef
	Wave ExactMassIsotopeAbund = root:HR_Sticks:ExactMassIsotopeAbund
	
	variable idex, num, numIsotopes, pt
	string isotopeStr, OrphanedIsotopes=""
	
	numIsotopes=0
	num = numpnts(ExactMassWave) 
	
	// first make 7 waves and fill in their values
	//1.04H fix - we make a new, "fresh" copy and then checks to see if either
	// (1) the user no longer wishes to constrain it or
	// (2) a new isotope species was added or removed.
	// This is somewhat klunky but there is no workaround for having an 'old' and a 'new' copy

	make/o/n=(num) root:HR_Sticks:NewIsotopicMassWave=0
	make/t/o/n=(num) root:HR_Sticks:NewIsotopicMassText =""
	make/t/o/n=(num) root:HR_Sticks:NewIsotopicParentText =""
	make/o/n=(num) root:HR_Sticks:NewIsotopicAbund=Nan
	make/o/n=(num) root:HR_Sticks:NewIsotopicMask=Nan	// mask flag indicating whether or not you want to fix this isotope
	make/o/n=(num) root:HR_Sticks:NewIsotopicMassPt=Nan	

	Wave NewIsotopicMassWave = root:HR_Sticks:NewIsotopicMassWave 
	Wave/T NewIsotopicMassText = root:HR_Sticks:NewIsotopicMassText
	Wave/T NewIsotopicParentText= root:HR_Sticks:NewIsotopicParentText
	Wave NewIsotopicAbund= root:HR_Sticks:NewIsotopicAbund
	Wave NewIsotopicMask= root:HR_Sticks:NewIsotopicMask
	Wave NewIsotopicMassPt= root:HR_Sticks:NewIsotopicMassPt
	// first, compile a list of isotope values, create default isotope mask
	
	for(idex=0; idex<num; idex+=1)
	
		if(strsearch(ExactMassText[idex], "j", 0,2) >= 0)		// we have an isotope
			NewIsotopicMassWave[numIsotopes]=ExactMassWave[idex]
			NewIsotopicMassText[numIsotopes]=ExactMassText[idex]
			NewIsotopicParentText[numIsotopes]=ExactMassIsotopeRef[idex]
			NewIsotopicAbund[numIsotopes]=ExactMassIsotopeAbund[idex]
			NewIsotopicMassPt[numIsotopes]=idex
	
			NewIsotopicMask[numIsotopes]= !(strsearch(ExactMassText[idex], "plus2",0)>0)
			
			numIsotopes+=1
		endif
	
	endfor

	DeletePoints numIsotopes, num, NewIsotopicMassWave, NewIsotopicMassText, NewIsotopicParentText
	DeletePoints numIsotopes, num, NewIsotopicAbund, NewIsotopicMask, NewIsotopicMassPt
	
//1.04H fix.  We now have a clean "new" copy.  Now we need to transfer the previous user settings (constrain or no) to the 'new' copy.
	Wave/z/T IsotopicMassText = root:HR_Sticks:IsotopicMassText
	Wave/z IsotopicMask= root:HR_Sticks:IsotopicMask

	if ( WaveExists(IsotopicMassText) && WaveExists(IsotopicMask) )
		num = numpnts(IsotopicMassText)		// number of isotopes in 'old' copy
		// for every 'old' isotopic mass, check to see if it in our new list and copy the constrain value
		for(idex=0; idex<num; idex+=1)
			isotopeStr = IsotopicMassText[idex]	// 'old' isotope
			pt =sq_findStr(NewIsotopicMassText,isotopeStr,1)
			if( pt >= 0)		// an isotope in the old wave also exists in the new wave
				NewIsotopicMask[pt]=IsotopicMask[idex]
			endif		
		endfor
	endif

	// now copy the 'new' into the 'old and kill 'new'
	duplicate/o NewIsotopicMassWave root:HR_Sticks:IsotopicMassWave
	duplicate/o NewIsotopicMassText root:HR_Sticks:IsotopicMassText
	duplicate/o NewIsotopicParentText root:HR_Sticks:IsotopicParentText
	duplicate/o NewIsotopicAbund root:HR_Sticks:IsotopicAbund
	duplicate/o NewIsotopicMask root:HR_Sticks:IsotopicMask	
	duplicate/o NewIsotopicMassPt root:HR_Sticks:IsotopicMassPt	

 	killwaves/z NewIsotopicMassWave, NewIsotopicMassText, NewIsotopicParentText
 	killwaves/z NewIsotopicAbund, NewIsotopicMask, NewIsotopicMassPt
		
	make/o/n=(num) root:HR_Sticks:ConstrainedKey/wave=ConstrainedKey
	HR_findConstrainedFlag(IsotopicMassText, IsotopicMask,ExactMassText, ConstrainedKey)

End


// fills in the values in wave IsotopicParentWave and IsotopicParentPt, to check for the existence of the isotopic parents in the chosen wave
// returns the string of isotope orphans
// 1.10D overhauled slightly due to issues with constraining.
// The waves in the HR_IsotopesTable are:
//	root:HR_Sticks:IsotopicMassWave
//	root:HR_Sticks:IsotopicMassText
//	root:HR_Sticks:IsotopicMask
//	root:HR_Sticks:IsotopicAbund
//	root:HR_Sticks:IsotopicParentText
// These waves have the same number of points, the total number of isotope HR ions that are fit, regardless as to whether they are constrained or not
// A subset of these waves which consist of the isotopes that are constrained are:

Function/S HR_CheckIsotopeParents([ChildMzLessThanParentFlag])  // ChildMzLessThanParentFlag added in 1.60D
variable ChildMzLessThanParentFlag
	
	Wave/T ExactMassText = root:HR_Sticks:ExactMassText		// names of all chosen masses 
	Wave ExactMassWave = root:HR_Sticks:ExactMassWave	// masses of all chosen masses 
	Wave/T ExactMassIsotopeRef = root:HR_Sticks:ExactMassIsotopeRef	// names of parents of all chosen masses (isotopic or no)
	Wave/T ExactMassText_all = root:HR:ExactMassText_all		// names of all chosen masses 
	Wave Mask_stick_integration = root:HR:Mask_stick_integration	// names of all chosen masses 

	Wave IsotopicMassWave = root:HR_Sticks:IsotopicMassWave 
	Wave/T IsotopicMassText = root:HR_Sticks:IsotopicMassText
	Wave IsotopicMask= root:HR_Sticks:IsotopicMask
	Wave/T IsotopicParentText = root:HR_Sticks:IsotopicParentText
	Wave IsotopicAbund= root:HR_Sticks:IsotopicAbund
	Wave IsotopicMassPt= root:HR_Sticks:IsotopicMassPt
	
	variable idex, nIsotopes, pt, isOK, nIsotopesConst
	string OrphanedIsotopes=""
		
	nIsotopes = numpnts(IsotopicMassWave)
	
//  duplicate all above, except for the IsotopicMask
	duplicate/o IsotopicMassWave 	root:HR_Sticks:IsotopicMassWaveConst/Wave=IsotopicMassWaveConst
	duplicate/o/t IsotopicMassText 	root:HR_Sticks:IsotopicMassTextConst/Wave=IsotopicMassTextConst
	duplicate/o/t IsotopicParentText root:HR_Sticks:IsotopicParentTextConst/Wave=IsotopicParentTextConst
	duplicate/o IsotopicAbund 	root:HR_Sticks:IsotopicAbundConst/Wave=IsotopicAbundConst
	duplicate/o IsotopicMassPt root:HR_Sticks:IsotopicMassPtConst/Wave=IsotopicMassPtConst

	// remove all HR ions that are NOT constrained,  are freely fitted.
	for (idex=nIsotopes-1;idex>=0;idex-=1)
		if (IsotopicMask[idex]==0)
			deletepoints idex, 1, IsotopicMassWaveConst, IsotopicMassTextConst, IsotopicParentTextConst, IsotopicAbundConst, IsotopicMassPtConst
		endif
	endfor

	nIsotopesConst = numpnts(IsotopicMassWaveConst)		// <= nIsotopes
	
	make/o/n=(nIsotopesConst) root:HR_Sticks:IsotopicParentWaveConst/Wave=IsotopicParentWaveConst=nan
	make/o/n=(nIsotopesConst) root:HR_Sticks:IsotopicParentPtConst/Wave=IsotopicParentPtConst=nan
	make/i/o/n=(nIsotopesConst) root:HR_Sticks:IsotopicParentWaveConstRnd/Wave=IsotopicParentWaveConstRnd

	// now check that the isotopic parent was also chosen as a mass to fit
	for(idex=0; idex<nIsotopesConst; idex+=1)			// check only all isotopic children that are constrained
		pt = sq_findStr(ExactMassText,IsotopicParentTextConst[idex],1)		// is case sensitive
		if(pt<0)		// parent NOT found
			pt = sq_findStr(ExactMassText_all,IsotopicParentTextConst[idex],1)		// is case sensitive
			if (pt>=0)		// parent exists, but not fit
				sq_butt_popThis("HRGrps_HR_AllMassesTable")
				ModifyTable/W=HR_AllMassesTable selection=(pt,0,pt,1,pt,1), topLeftCell=(pt, 0)
				DoAlert, 2,  "Isotopic parent HR ion "+IsotopicParentTextConst[idex]+" is not fit for isotope "+IsotopicMassTextConst[idex]+" at m/z "+num2str(IsotopicMassWaveConst[idex])+" To choose to fit the parent, and try again press yes. To not constrain the isotope press No."
				if (V_flag==1)			// User wishes to fit the parent and continue.
					Mask_stick_integration[pt]=1
					HR_butt_MakeExactMassWaves("")
				elseif (V_flag==3)
					DoAlert 0,"The user chose to quit. Aborting from HR_CheckIsotopeParents"
				elseif (V_flag==2)
					IsotopicMask[idex]=0
//					FindValue/Text=IsotopicMassTextConst[idex]/TXOP=4  ExactMassText_all 
//					Mask_stick_integration[V_Value]=0
					continue
				endif
			elseif (strlen(IsotopicParentTextConst[idex])==0)		//1.19F
				sq_WindowIsToBePopped("HR_AllMassesTable")
				FindValue/Text=IsotopicMassTextConst[idex]/TXOP=4  ExactMassText_all 
				ModifyTable/W=HR_AllMassesTable topLeftCell=(V_Value ,0)
				abort "Isotopic parent of HR ion "+IsotopicMassTextConst[idex]+" is blank. Is there a typo? Aborting from HR_CheckIsotopeParents" // 1.14E				
			else		// parent value doesn't exist at all 
				sq_WindowIsToBePopped("HR_AllMassesTable")
				FindValue/Text=IsotopicMassTextConst[idex]/TXOP=4  ExactMassText_all 
				ModifyTable/W=HR_AllMassesTable topLeftCell=(V_Value ,0)
				abort "Isotopic parent HR ion "+IsotopicParentTextConst[idex]+" of "+IsotopicMassTextConst[idex]+" does not exist in the list of all possible HR ions. Is there a typo? Aborting from HR_CheckIsotopeParents" // 1.14E
			endif
//		elseif( pt <0)	
//			OrphanedIsotopes+= IsotopicMassTextConst[idex] + " "+num2str(IsotopicMassWaveConst[idex]) + ";"
		else
			IsotopicParentWaveConst[idex]=ExactMassWave[pt]
			IsotopicParentPtConst[idex] = pt
		endif	
	endfor

	IsotopicParentWaveConstRnd = round(IsotopicParentWaveConst)
	
	make/i/o/n=(nIsotopesConst) root:HR_Sticks:IsotopicChildWaveConstRnd/Wave=IsotopicChildWaveConstRnd
	make/i/o/n=(nIsotopesConst) root:HR_Sticks:isotopicChildWaveConstRndRev/Wave=isotopicChildWaveConstRndRev

	IsotopicChildWaveConstRnd = round(IsotopicMassWaveConst)
	isotopicChildWaveConstRndRev = round(IsotopicMassWaveConst)
	Sort/R isotopicChildWaveConstRndRev isotopicChildWaveConstRndRev	// reverse sort	// we don't care about doubles, triples, etc.

	return OrphanedIsotopes

End


// Determine the order of integer mass values to fit, based on isotopic parent values, and isotopic mask values
// Creates two waves: MassFitOrder and IsotopePairings
//  called from HR_CreateHoldConstraintWaves(), HR_butt_CheckIsotopeMask()
// overhauled in 1.20D so that metal isotopes are unconstrained by default and the mass fitting order is always numerically increasing.
Function HR_SortIsotopicValues()
	
	wave ExactMassWave = root:HR_Sticks:ExactMassWave	

	wave IsotopicMassWaveConst = root:HR_Sticks:IsotopicMassWaveConst		// created in HR_CheckIsotopeParents()	
	wave IsotopicParentWaveConstRnd = root:HR_Sticks:IsotopicParentWaveConstRnd  // created in HR_CheckIsotopeParents()	

	wave IsotopicChildWaveConstRnd = root:HR_Sticks:IsotopicChildWaveConstRnd		// created in HR_CheckIsotopeParents()	
	wave isotopicChildWaveConstRndRev= root:HR_Sticks:isotopicChildWaveConstRndRev  // created in HR_CheckIsotopeParents()	

	string MassDoneList=""	// a list of mass values, in order or execution.  Initialized to amus that have no isotopes
	string MassToCheckList=""	// list of masses to check
	string bad2goodindexList=""		// a temporary list of values going from MassToCheckList to MassDoneList

	variable idex, jdex, num, beginPt, endPt, isOK, currentmz, nIsotopesConst
	variable minInteger, maxInteger, numToCheck

	wavestats/q/M=1 ExactMassWave
	minInteger = round(V_min)
	maxInteger = round(V_max)
	
	// initialize a list of integer values of masses, in the order in which they should be found.
	// then remove all our isotope mass, but only if the mask value indicatest that the isotope will be constrained
	for(idex=minInteger; idex<=maxInteger; idex+=1)
		MassDoneList+=num2str(idex)+";"
	endfor
	
	nIsotopesConst=numpnts(IsotopicMassWaveConst)		

	make/t/o/n=(maxInteger-minInteger+1) root:HR_Sticks:IsotopePairings = ""
	make/o/n=(maxInteger-minInteger+1) root:HR_Sticks:MassFitOrder = nan		// to be filled in at the end
	wave/t IsotopePairings = root:HR_Sticks:IsotopePairings
	wave MassFitOrder= root:HR_Sticks:MassFitOrder

	variable FitOrderIsIncreasing = 1			//1.20D

	if (FitOrderIsIncreasing==0)		// the old way
		for(idex=0; idex<nIsotopesConst; idex+=1)
	
			currentmz = IsotopicChildWaveConstRnd[idex]
			if (WhichListItem(num2str(currentmz), MassToCheckList)<0)		// we need to constrain and it isn't in our list yet
				MassToCheckList+= num2str(currentmz)+";"	
				MassDoneList=RemoveFromList(num2str(currentmz), MassDoneList)
			endif
		
		endfor
	
		// now begin the process of figuring out which of the constrained isotopes needs to be calculated first because they are parents themselves	
		// start categorizing
		// the goal is to eliminate entries in the MassToCheckList and to put them in the MassDoneList
		do	 // while numToCheck>0 or we are at a stalemate
			
			numToCheck=ItemsInList(MassToCheckList)
			bad2goodindexList=""
			
			for(idex=0; idex<numToCheck; idex+=1)		// this for loops runs through all the masses that need to be checked
				
				currentmz = str2num(StringFromList(idex, MassToCheckList)) 
				// find out if there is more than one isotope at this mass		
				FindValue/I=(currentmz)  IsotopicChildWaveConstRnd
				beginPt=V_value
				FindValue/I=(currentmz)  isotopicChildwaveConstRndRev
				endPt=nIsotopesConst-V_Value-1
				
				isOK=1		// a boolean to indicate whether all the parents of this mass have already been put in the 'good' list - have already been found
				for(jdex=beginPt;jdex<=endPt; jdex+=1)
					isOK *= (FindListItem(num2str(IsotopicParentWaveConstRnd[jdex]),MassDoneList ) >=0)
				endfor
				if (isOK)
					bad2goodindexList+=num2str(currentmz)+";"			
				endif
							
			endfor
	
			num=ItemsInList(bad2goodindexList)		// this for loop runs through all the masses to be put in the 'good' list
			for(idex=0;idex<num; idex+=1)
				currentmz = str2num(StringFromList(idex, bad2goodindexList)) 
				// remove from masses to check
				MassToCheckList= RemoveFromList(num2str(currentmz), MassToCheckList)
				// add to good masses
				MassDoneList += num2str(currentmz)+";"
			endfor	
	
		while(ItemsInList(MassToCheckList)!=numToCheck)
		
		if ( ItemsInList(MassToCheckList)!=0 )
			currentmz = str2num(StringFromList(0,MassToCheckList))
			idex =BinarySearch(root:HR_sticks:IsotopicMassWave, currentmz)
			sq_butt_popThis("HRGrps_HR_IsotopesTable")
			ModifyTable/W=HR_IsotopesTable selection=(idex-1, 0 ,idex-1,2,idex-1,2), topLeftCell=(idex-1, 0)  //1.15A
			Abort "There is a problem resolving the mass order in which todo the HR fits because of isotope parent conflicts with: "+MassToCheckList +" Aborting from  HR_SortIsotopicValues"
		endif
		
		// lame way to create MassFitOrder, because sq_List2TextWave isn't data folder aware, and we want a numerical wave, not a text wave.
		pk_List2TextWave(MassDoneList, "tempFitOrder")
		wave/t tempFitOrder = root:tempFitOrder
		MassFitOrder = str2num(tempFitOrder[p])
		killwaves/z root:tempFitOrder	

	else  // (FitOrderIsIncreasing==0)		// the old way
		MassFitOrder = minInteger+p
		HR_unConstrainChildLTParent()
	endif // (FitOrderIsIncreasing==0)		// the old way


//	IsotopePairings =  HR_MakeIsotopePairings(MassFitOrder[p])		//1.08E
	HR_MakeIsotopePairings()		//1.08E
	
End

// new in 1.60D, modified in 1.60S
Function HR_unConstrainChildLTParent()

	wave/t IsotopicMassText = root:HR_Sticks:IsotopicMassText
	wave IsotopicMassWave = root:HR_Sticks:IsotopicMassWave
	wave IsotopicMask = root:HR_Sticks:IsotopicMask
	wave IsotopicAbund = root:HR_Sticks:IsotopicAbund
	wave/t IsotopicParentText = root:HR_Sticks:IsotopicParentText

	wave/t ExactMassText_all = root:HR:ExactMassText_all
	wave/d ExactMassWave_all = root:HR:ExactMassWave_all

	variable idex, num, IsoChildMass, IsoParentMass, counter=0
	string IsoChild, IsoParent, changedList=""
	
	for (idex=0;idex<numpnts(IsotopicMask);idex+=1)
		IsoParent = IsotopicParentText[idex]
		IsoChildMass = IsotopicMassWave[idex]
		FindValue/TXOP=4/TEXT=IsotopicParentText[idex] ExactMassText_all
		if (V_Value>=0 && IsoChildMass < ExactMassWave_all[V_Value] )
			if (IsotopicMask[idex])
				IsotopicMask[idex]=0
				counter+=1
				changedList+=IsotopicMassText[idex]+";"
			endif
		endif		
	endfor

	if (counter >0)
		print "// Be aware that Pika automatically changed the constraint settings for these isotopic children that have parents with a mass greater than then child:"+ changedList
	endif

End


// returns a string containing groups of 3 items:  child point number, parent point number, and the specific mass position within that amu
// but only for species we are constraining.  Else returns the empty string.
//  called from HR_SortIsotopicValues()
Function HR_MakeIsotopePairings() 

	wave/t IsotopePairings = root:HR_Sticks:IsotopePairings		// results of this function
	wave MassFitOrder = root:HR_Sticks:MassFitOrder	// set in HR_SortIsotopicValues()
		
	wave IsotopicMassWaveConst= root:HR_Sticks:IsotopicMassWaveConst		// created in HR_CheckIsotopeParents()
	wave/t IsotopicMassTextConst= root:HR_Sticks:IsotopicMassTextConst		// created in HR_CheckIsotopeParents()	
	wave IsotopicParentWaveConst= root:HR_Sticks:IsotopicParentWaveConst		// created in HR_CheckIsotopeParents()	
	wave IsotopicChildWaveConstRnd= root:HR_Sticks:IsotopicChildWaveConstRnd	// created in HR_CheckIsotopeParents()	
	wave isotopicChildWaveConstRndRev = root:HR_Sticks:isotopicChildWaveConstRndRev	// created in HR_CheckIsotopeParents()	
	wave IsotopicParentWaveConstRnd= root:HR_Sticks:IsotopicParentWaveConstRnd	// created in HR_CheckIsotopeParents()	
	wave IsotopicParentPtConst = root:HR_Sticks:IsotopicParentPtConst			// created in HR_CheckIsotopeParents()	
	wave IsotopicMassPtConst = root:HR_Sticks:IsotopicMassPtConst				// created in HR_CheckIsotopeParents()	
	
	// first create unitMass wave
	 wave ExactMassWave= root:HR_sticks:ExactMassWave
	wave/t ExactMassText= root:HR_Sticks:ExactMassText

	HR_MakeUnitMassWave()
		
	wave numOfPeaks = root:HR_Sticks:numOfPeaks
	wave UnitMass = root:HR_Sticks:UnitMass	

	variable beginPT, endPt, idex, nIsotopes, mzdex, num, currentmz
	variable AMUthPos, kdex,  MzNumPeaks, beginMz, MzStartPt
	string IsotopicStr
	string returnStr = ""
		
	num=numpnts(MassFitOrder)
	nIsotopes = numpnts(IsotopicMassWaveConst)
	
	for(mzdex=0;mzdex<num;mzdex+=1)
	
		currentmz = MassFitOrder[mzdex]
		FindValue/I=(currentmz)  IsotopicChildWaveConstRnd
		
		if (V_value==-1)	
			IsotopePairings[mzdex]=""
			continue
		endif	
		
		// it is an isotope	
		returnStr=""		//1.08E
		
		beginPt=V_value
		FindValue/I=(currentmz)  isotopicChildwaveConstRndRev
		endPt=nIsotopes-V_Value-1
		
		beginMz = UnitMass[0]
		
	//	check all isotopes at this mz
		for (idex=beginPt; idex<=endPt; idex+=1)
			returnStr +=num2str(IsotopicMassPtConst[idex]) +"/"+num2str(IsotopicParentPtConst[idex])  +"/"
			// find which ith position at the mz this isotope lives
			IsotopicStr=IsotopicMassTextConst[idex]
			MzNumPeaks = numOfPeaks[CurrentMz - beginMz]
			MzStartPt = ( CurrentMz !=beginMz ? Sum(NumOfPeaks, 0,CurrentMz -1 - beginMz) :0 )// gives the sum total of all masses fit up to this point
			kdex=0
			do
				if(stringMatch(IsotopicStr,ExactMassText[MzStartPt+kdex])>0)
					AMUthPos=kdex
					break
				endif
				kdex+=1
			while(kdex<MzNumPeaks)
			
			returnStr +=num2str(AMUthPos) +";"
		endfor	
			
		IsotopePairings[mzdex]= returnStr
	
	endfor

End


// will start everything up
// revamped in 1.09H
Function PK_SetUpParamPanel()

	NewDataFolder/o root:HR_ParamPanel
	
	sq_initializeGlobalNVar("root:HR_ParamPanel:HR_TimeParamRunNum", 0)
	sq_initializeGlobalNVar("root:HR_ParamPanel:HR_TimeParamCursorPoint", 0)
	sq_initializeGlobalNVar("root:HR_ParamPanel:HR_TimeParamSeriesMzStart",12)		 // 12 is arbitrary
	sq_initializeGlobalNVar("root:HR_ParamPanel:HR_TimeParamSeriesOpen", 0)
	sq_initializeGlobalNVar("root:HR_ParamPanel:HR_TimeParamSeriesClosed", 0)
	sq_initializeGlobalNVar("root:HR_ParamPanel:HR_TimeParamSeriesDiff", 1)
	sq_initializeGlobalNVar("root:HR_ParamPanel:HR_TimeParamSqBaseline", 0)
	sq_initializeGlobalNVar("root:HR_ParamPanel:HR_TimeParamPikaBaseline", 0)
	sq_initializeGlobalNVar("root:HR_ParamPanel:HR_TimeParamPikaSum", 1)
	sq_initializeGlobalNVar("root:HR_ParamPanel:HR_TimeParamOrigData", 1)
	sq_initializeGlobalNVar("root:HR_ParamPanel:HR_TimeParamPercentData", 0)

	nvar HR_TimeParamSeriesMzStart = root:HR_ParamPanel:HR_TimeParamSeriesMzStart
	nvar HR_TimeParamSeriesOpen = root:HR_ParamPanel:HR_TimeParamSeriesOpen
	nvar HR_TimeParamSeriesClosed = root:HR_ParamPanel:HR_TimeParamSeriesClosed
	nvar HR_TimeParamSeriesDiff = root:HR_ParamPanel:HR_TimeParamSeriesDiff
	nvar HR_TimeParamPikaBaseline = root:HR_ParamPanel:HR_TimeParamPikaBaseline
	nvar HR_TimeParamPikaSum = root:HR_ParamPanel:HR_TimeParamPikaSum
	nvar HR_TimeParamSqBaseline = root:HR_ParamPanel:HR_TimeParamSqBaseline	
	nvar HR_TimeParamOrigData = root:HR_ParamPanel:HR_TimeParamOrigData
	nvar HR_TimeParamPercentData = root:HR_ParamPanel:HR_TimeParamPercentData

	wave UnitMass = root:HR_sticks:UnitMass //1.20C
	string specType="", dataType="", dataType2 = "", mzList =""	
	variable idex, num=10		// 10 is arbitrary - the user can look at 10 values per graph

	if (HR_TimeParamSeriesOpen)
		specType="Open"
	elseif(HR_TimeParamSeriesClosed)
		specType="Closed"
	else	
		specType="Diff"
	endif
	
	dataType2="MSS"
	if (HR_TimeParamPikaSum)
		dataType="ResidSum"
	elseif(HR_TimeParamPikaBaseline)
		dataType="HRBk"
	else	
		dataType="MSS" + specType+"BaseL_p"		// these were not named ideally		
		dataType2 = "MSS"+ specType
		specType = ""
	endif

	if (FindDimLabel(root:index:squirrel_index,2,dataType+SpecType) <0)
		abort "The data set "+dataType+SpecType +" doesn't exist.  Perhaps you need to do Step 5? - aborting from PK_SetUpParamPanel"
	endif
	
	HR_TimeParamSeriesMzStart = max(WaveMin(UnitMass), HR_TimeParamSeriesMzStart)
	HR_TimeParamSeriesMzStart = min(WaveMax(UnitMass), HR_TimeParamSeriesMzStart)
	
	for (idex = 0;idex < num; idex+=1)
		mzList +=num2str(HR_TimeParamSeriesMzStart + idex) + "/"
	endfor	
	
	svar todoStr=root:panel:sq_toDoWvNm		//^ 1.35 change
	wave/z todo=$(todoStr)	

	squirrel_fetch(todo, PK_ExtractParamSum, "MzVal:"+mzList+";dest:HRParam_Trace_",dataType+SpecType+";"+dataType2+SpecType+"_p" )
	
	PK_ConvertParam(1)		// 1.03E
	DoWindow/F HRParamPanel
	if (V_flag==0)
		PK_CreateHRParamPanel()
	endif
	
End


//  Very simple for converting  param waves  to percent Sq sticks
Function PK_ConvertParam(OriginalRetrievalFlag)
variable OriginalRetrievalFlag

	wave MSSDiff_p = root:MSSDiff_p
	nvar HR_TimeParamPercentData = root:HR_ParamPanel:HR_TimeParamPercentData
	nvar HR_TimeParamOrigData = root:HR_ParamPanel:HR_TimeParamOrigData
	nvar HR_TimeParamSeriesMzStart = root:HR_ParamPanel:HR_TimeParamSeriesMzStart
	
	variable jdex, numOfMzs=10
				
	for (jdex=0;jdex<numOfMzs;jdex+=1)
		wave destwave =  $("root:HR_ParamPanel:HRParam_Trace_"+HR_gen_Num2Str2Char(jdex))
		wave Sqwave =  $("root:HR_ParamPanel:HRParam_SqDiff_"+HR_gen_Num2Str2Char(jdex))
		if (HR_TimeParamPercentData)
			destWave/=Sqwave[p]
		elseif(HR_TimeParamOrigData && !OriginalRetrievalFlag)
			destWave*=Sqwave[p]		
		endif
	endfor
		
End



//  Very simple squirrel fetch function
Function PK_ExtractParamSum(index_pos,paramstr,acorn,n_acorns)
	wave index_pos
	string paramstr
	variable acorn 
	variable n_acorns

	string dest = stringbykey("dest",paramstr)		// note that this only contains the prefix to the destination wave
	wave ParamMatrix=$stringbykey("acorn_0",paramstr)
	wave MSS_p=$stringbykey("acorn_1",paramstr)
	string MzValList = stringbykey("MzVal",paramstr)
	wave series_index = root:index:series_index
	wave rn_series = root:index:rn_series
	wave UnitMass = root:HR_sticks:UnitMass
	nvar HR_TimeParamSqBaseline = root:HR_ParamPanel:HR_TimeParamSqBaseline
	
	variable idex, numOfRuns = numpnts(index_pos), currentRunIndex, RunVar, unitMassOffset
	variable totalNumRuns, jdex, thisMz, numOfMzs = ItemsinList(MzValList, "/")
	
	totalNumRuns = numpnts(series_index)
	
	if (HR_TimeParamSqBaseline)
		unitMassOffset=1		// becasue we count from 0
	else
		unitMassOffset=UnitMass[0]	
	endif
	
	if (acorn==0)
		for (jdex=0;jdex<10;jdex+=1)
			make/o/n=(totalNumRuns) $("root:HR_ParamPanel:"+dest+HR_gen_Num2Str2Char(jdex))=nan
			make/o/n=(totalNumRuns)  $("root:HR_ParamPanel:HRParam_SqDiff_"+HR_gen_Num2Str2Char(jdex))=nan
		endfor
	endif
			
	for (idex=0;idex<numOfRuns;idex+=1)
	
		sq_ams_prog_upd(100*(acorn+(idex/numOfRuns))/n_acorns)
		
		if (index_pos[idex]==-1)
			continue
		endif
		
		// first get all info that is constant for this run
		currentRunIndex = series_index[index_pos[idex]]				
		RunVar = rn_series[currentRunIndex]		// not necessary, just updates panel
					
		for (jdex=0;jdex<numOfMzs;jdex+=1)
			thisMz = str2num(stringfromList(jdex, MzValList, "/"))
			wave destwave = $("root:HR_ParamPanel:"+dest+HR_gen_Num2Str2Char(jdex))
			wave SQwave = $("root:HR_ParamPanel:HRParam_SqDiff_"+HR_gen_Num2Str2Char(jdex))
			destwave[currentRunIndex] = ParamMatrix[idex][thisMz-unitMassOffset]
			SQwave[currentRunIndex] = MSS_p[idex][thisMz-1]
		endfor
			
	endfor // each run	

End


Function PK_CreateHRParamPanel()

	Display/N=HRParamPanel /W=(100,100,800,600)  as "HRParamPanel"
	
	AppendToGraph/L=Pair0Left root:HR_ParamPanel:HRParam_Trace_00 vs root:index:t_series; 
	AppendToGraph/R=Pair0Right root:HR_ParamPanel:HRParam_Trace_01 vs root:index:t_series; 
	AppendToGraph/L=Pair1Left :HR_ParamPanel:HRParam_Trace_02 vs root:index:t_series; 
	AppendToGraph/R=Pair1Right root:HR_ParamPanel:HRParam_Trace_03 vs root:index:t_series; 
	AppendToGraph/L=Pair2Left :HR_ParamPanel:HRParam_Trace_04 vs root:index:t_series; 
	AppendToGraph/R=Pair2Right :HR_ParamPanel:HRParam_Trace_05 vs root:index:t_series
	AppendToGraph/L=Pair3Left :HR_ParamPanel:HRParam_Trace_06 vs root:index:t_series; 
	AppendToGraph/R=Pair3Right :HR_ParamPanel:HRParam_Trace_07 vs root:index:t_series
	AppendToGraph/L=Pair4Left :HR_ParamPanel:HRParam_Trace_08 vs root:index:t_series; 
	AppendToGraph/R=Pair4Right :HR_ParamPanel:HRParam_Trace_09 vs root:index:t_series
	Label bottom " "
	ModifyGraph dateInfo(bottom)={0,1,0}
	ModifyGraph axisEnab(Pair0Left)={0.01,0.19},axisEnab(Pair0Right)={0.01,0.19}
	ModifyGraph axisEnab(Pair1Left)={0.21,0.39},axisEnab(Pair1Right)={0.21,0.39}
	ModifyGraph axisEnab(Pair2Left)={0.41,0.59},axisEnab(Pair2Right)={0.41,0.59}
	ModifyGraph axisEnab(Pair3Left)={0.61,0.79},axisEnab(Pair3Right)={0.61,0.79}
	ModifyGraph axisEnab(Pair4Left)={0.81,0.99},axisEnab(Pair4Right)={0.81,0.99}
	ModifyGraph freePos(Pair0Left)={0.05,kwFraction}, freePos(Pair0Right)={0.05,kwFraction}
	ModifyGraph freePos(Pair1Left)={0.05,kwFraction}, freePos(Pair1Right)={0.05,kwFraction}
	ModifyGraph freePos(Pair2Left)={0.05,kwFraction}, freePos(Pair2Right)={0.05,kwFraction}
	ModifyGraph freePos(Pair3Left)={0.05,kwFraction}, freePos(Pair3Right)={0.05,kwFraction}
	ModifyGraph freePos(Pair4Left)={0.05,kwFraction}, freePos(Pair4right)={0.05,kwFraction}
	
	ModifyGraph axisEnab(bottom)={0.05,0.95}
	ModifyGraph zero=1
	SetAxis/A/E=2 		// symmetric about 0
	ModifyGraph mode=4,msize=1
	ModifyGraph lsize=2
	ShowInfo

	ModifyGraph rgb(HRParam_Trace_00)=(65280,16384,55552),rgb(HRParam_Trace_01)=(34952,21074,34952)
	ModifyGraph rgb(HRParam_Trace_02)=(49858,49858,34952),rgb(HRParam_Trace_03)=(0,60652,60652)
	ModifyGraph rgb(HRParam_Trace_04)=(0,0,63222),rgb(HRParam_Trace_05)=(0,51400,0),rgb(HRParam_Trace_06)=(65535,65535,0)
	ModifyGraph rgb(HRParam_Trace_08)=(49344,0,0),rgb(HRParam_Trace_09)=(39321,21845,51657)

	ControlBar 75
	GroupBox groupDataType,pos={5,5},size={110,65},title="Data type"
//	CheckBox check_PikaBaseline,pos={10,37},size={94,15},proc=PK_ck_ParamDataType,title="Old Pika Baseline"		// 1.04P
//	CheckBox check_PikaBaseline,variable= root:HR_ParamPanel:HR_TimeParamPikaBaseline,mode=1
	CheckBox check_Residual,pos={10,22},size={97,15},proc=PK_ck_ParamDataType,title="Residual Sum"
	CheckBox check_Residual,variable= root:HR_ParamPanel:HR_TimeParamPikaSum,mode=1
	CheckBox check_SqBaseline,pos={10,52},size={85,15},proc=PK_ck_ParamDataType,title="Sq Baseline"
	CheckBox check_SqBaseline,variable= root:HR_ParamPanel:HR_TimeParamSqBaseline,mode=1

	GroupBox groupSpectraType,pos={120,5},size={80,65},title="Spectra type"
	CheckBox check_diff,pos={129,19},size={36,15},proc=PK_ck_ParamSpectraType,title="Diff"
	CheckBox check_diff,variable= root:HR_ParamPanel:HR_TimeParamSeriesDiff,mode=1
	CheckBox check_open,pos={129,35},size={48,15},proc=PK_ck_ParamSpectraType,title="Open"
	CheckBox check_open,variable= root:HR_ParamPanel:HR_TimeParamSeriesOpen,mode=1
	CheckBox check_closed,pos={129,50},size={58,15},proc=PK_ck_ParamSpectraType,title="Closed"
	CheckBox check_closed,variable= root:HR_ParamPanel:HR_TimeParamSeriesClosed,mode=1

	GroupBox groupMathType,pos={205,4},size={90,65},title="Math type"
	CheckBox check_DataOriginal,pos={210,24},size={61,15},proc=PK_ck_ParamDataMath,title="Original"
	CheckBox check_DataOriginal,variable= root:HR_ParamPanel:HR_TimeParamOrigData,mode=1
	CheckBox check_percentSq,pos={210,43},size={75,15},proc=PK_ck_ParamDataMath,title="% Sq stick"
	CheckBox check_percentSq,variable= root:HR_ParamPanel:HR_TimeParamPercentData,mode=1

	GroupBox groupMzStart,pos={300,5},size={160,65},title="Wave mz thru mz+9"
	SetVariable setvarMzStart,pos={307,25},size={140,18},proc=PK_setVar_changeMz,title="Beginning mz"
	SetVariable setvarMzStart,limits={1,inf,1}, value= root:HR_ParamPanel:HR_TimeParamSeriesMzStart
	Button buttonminus10,pos={332,44},size={30,20},proc=PK_butt_delta10,title="<<"
	Button buttonplus10,pos={386,44},size={30,20},proc=PK_butt_delta10,title=">>"

	GroupBox groupCsrPlacement,pos={465,5},size={200,65},title="Set all cursors to same run"
	SetVariable setvarCsrPt,pos={471,23},size={180,18},proc=PK_setVar_changeCsrPt,title="Cursor Point number"
	SetVariable setvarCsrPt,limits={0,inf,1},value= root:HR_ParamPanel:HR_TimeParamCursorPoint
	SetVariable setvarRunNum,pos={471,45},size={151,18},proc=PK_setVar_changeCsrPt,title="Run number:"
	SetVariable setvarRunNum,limits={0,inf,0},value= root:HR_ParamPanel:HR_TimeParamRunNum,noedit= 1

	GroupBox groupCsrInfo,pos={670,5},size={245,65},title="Print cursor info to history"
	PopupMenu popup_ParamGraph_csr,pos={823,21},size={87,23},title="Cursor"
	PopupMenu popup_ParamGraph_csr,mode=1,popvalue="A",value= #"\"A;B;C;D;E;F;G;;H;I;J\""
	Button butt_PrintAllCrsrVals,pos={678,46},size={190,20},proc=PK_Param_PrintCursorVals,title="Print All Cursor Values to History"
	Button butt_PrintCrsrVal,pos={678,21},size={140,20},proc=PK_Param_PrintCursorVals,title="Print Cursor to History ->"

	PK_AddLabelsToAxis()
	PK_setVar_changeCsrPt("",nan,"","")

End


Function PK_AddLabelsToAxis()

	nvar HR_TimeParamSeriesMzStart = root:HR_ParamPanel:HR_TimeParamSeriesMzStart

	DoWindow/F HRParamPanel
	
	Label Pair0left "\\s(HRParam_Trace_00)"+num2str(HR_TimeParamSeriesMzStart)	
	Label Pair0right "\\s(HRParam_Trace_01)"+num2str(HR_TimeParamSeriesMzStart+1)
	Label Pair1left "\\s(HRParam_Trace_02)"+num2str(HR_TimeParamSeriesMzStart+2)	
	Label Pair1right "\\s(HRParam_Trace_03)"+num2str(HR_TimeParamSeriesMzStart+3)
	Label Pair2left "\\s(HRParam_Trace_04)"+num2str(HR_TimeParamSeriesMzStart+4)	
	Label Pair2right "\\s(HRParam_Trace_05)"+num2str(HR_TimeParamSeriesMzStart+5)
	Label Pair3left "\\s(HRParam_Trace_06)"+num2str(HR_TimeParamSeriesMzStart+6)	
	Label Pair3right "\\s(HRParam_Trace_07)"+num2str(HR_TimeParamSeriesMzStart+7)
	Label Pair4left "\\s(HRParam_Trace_08)"+num2str(HR_TimeParamSeriesMzStart+8)	
	Label Pair4right "\\s(HRParam_Trace_09)"+num2str(HR_TimeParamSeriesMzStart+9)
	
	ModifyGraph lblPos=60,lblLatPos=0

End


Function PK_butt_delta10(ctrlName) : ButtonControl
	String ctrlName

	nvar HR_TimeParamSeriesMzStart = root:HR_ParamPanel:HR_TimeParamSeriesMzStart

	if (strsearch(ctrlName, "plus", 0)>=0)
		HR_TimeParamSeriesMzStart = min(HR_TimeParamSeriesMzStart+10,300)
	else
		HR_TimeParamSeriesMzStart = max(HR_TimeParamSeriesMzStart-10,10)
	endif
	
	PK_SetUpParamPanel()
	PK_AddLabelsToAxis()

End


Function PK_setVar_changeMz(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum
	String varStr
	String varName	
	
	PK_SetUpParamPanel()
	PK_AddLabelsToAxis()
	
End


Function PK_setVar_changeCsrPt(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum
	String varStr
	String varName	
	
	nvar  HR_TimeParamCursorPoint =  root:HR_ParamPanel:HR_TimeParamCursorPoint
	nvar HR_TimeParamRunNum =  root:HR_ParamPanel:HR_TimeParamRunNum

	cursor A HRParam_Trace_00 HR_TimeParamCursorPoint
	cursor B HRParam_Trace_01 HR_TimeParamCursorPoint
	cursor C HRParam_Trace_02 HR_TimeParamCursorPoint
	cursor D HRParam_Trace_03 HR_TimeParamCursorPoint
	cursor E HRParam_Trace_04 HR_TimeParamCursorPoint
	cursor F HRParam_Trace_05 HR_TimeParamCursorPoint
	cursor G HRParam_Trace_06 HR_TimeParamCursorPoint
	cursor H HRParam_Trace_07 HR_TimeParamCursorPoint
	cursor I HRParam_Trace_08 HR_TimeParamCursorPoint
	cursor J HRParam_Trace_09 HR_TimeParamCursorPoint

	wave rn_series = root:index:rn_series
	HR_TimeParamRunNum = rn_series[HR_TimeParamCursorPoint]

End	
	

Function PK_ck_ParamSpectraType(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked

	nvar HR_TimeParamSeriesOpen = root:HR_ParamPanel:HR_TimeParamSeriesOpen
	nvar HR_TimeParamSeriesClosed = root:HR_ParamPanel:HR_TimeParamSeriesClosed
	nvar HR_TimeParamSeriesDiff = root:HR_ParamPanel:HR_TimeParamSeriesDiff

	strswitch(ctrlName)
		case "check_open": // mouse up
			HR_TimeParamSeriesOpen = 1
			HR_TimeParamSeriesClosed = 0
			HR_TimeParamSeriesDiff = 0
			break
		case "check_closed": // mouse up
			HR_TimeParamSeriesOpen = 0
			HR_TimeParamSeriesClosed = 1
			HR_TimeParamSeriesDiff = 0
			break
		case "check_diff": // mouse up
			HR_TimeParamSeriesOpen = 0
			HR_TimeParamSeriesClosed = 0
			HR_TimeParamSeriesDiff = 1
			break
	endswitch

	PK_SetUpParamPanel()
	PK_AddLabelsToAxis()

End


Function PK_ck_ParamDataType(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked

	nvar HR_TimeParamSqBaseline = root:HR_ParamPanel:HR_TimeParamSqBaseline
	nvar HR_TimeParamPikaBaseline = root:HR_ParamPanel:HR_TimeParamPikaBaseline
	nvar HR_TimeParamPikaSum = root:HR_ParamPanel:HR_TimeParamPikaSum

	strswitch(ctrlName)
//		case "check_PikaBaseline": // mouse up  // 1.16Z
//			HR_TimeParamPikaBaseline = 1
//			HR_TimeParamSqBaseline = 0
//			HR_TimeParamPikaSum = 0
//			break
		case "check_SqBaseline": // mouse up
			HR_TimeParamPikaBaseline = 0
			HR_TimeParamSqBaseline = 1
			HR_TimeParamPikaSum = 0
			break
		case "check_Residual": // mouse up
			HR_TimeParamPikaBaseline = 0
			HR_TimeParamSqBaseline = 0
			HR_TimeParamPikaSum = 1
			break
	endswitch

	PK_SetUpParamPanel()
	PK_AddLabelsToAxis()

End


Function PK_ck_ParamDataMath(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked

	nvar HR_TimeParamOrigData = root:HR_ParamPanel:HR_TimeParamOrigData
	nvar HR_TimeParamPercentData = root:HR_ParamPanel:HR_TimeParamPercentData

	strswitch(ctrlName)
		case "check_DataOriginal": // mouse up
			HR_TimeParamOrigData = 1
			HR_TimeParamPercentData = 0
			break
		case "check_percentSq": // mouse up
			HR_TimeParamOrigData = 0
			HR_TimeParamPercentData = 1
			break
	endswitch

	PK_ConvertParam(0)

End


Function PK_Param_PrintCursorVals(ctrlName) : ButtonControl
	String ctrlName

	nvar HR_TimeParamSeriesOpen = root:HR_ParamPanel:HR_TimeParamSeriesOpen
	nvar HR_TimeParamSeriesClosed = root:HR_ParamPanel:HR_TimeParamSeriesClosed
	nvar HR_TimeParamSeriesDiff = root:HR_ParamPanel:HR_TimeParamSeriesDiff
	nvar HR_TimeParamSeriesMzStart = root:HR_ParamPanel:HR_TimeParamSeriesMzStart
	nvar HR_TimeParamSqBaseline = root:HR_ParamPanel:HR_TimeParamSqBaseline
	nvar HR_TimeParamPikaBaseline = root:HR_ParamPanel:HR_TimeParamPikaBaseline
	nvar HR_TimeParamPikaSum = root:HR_ParamPanel:HR_TimeParamPikaSum
	nvar HR_TimeParamOrigData = root:HR_ParamPanel:HR_TimeParamOrigData
	nvar HR_TimeParamPercentData = root:HR_ParamPanel:HR_TimeParamPercentData

	wave rn_series = root:index:rn_series 
	string SpecType="", dataType = "", mathType=""
	string tempStr, CsrStr, CsrInfoList, CursorList = "A;B;C;D;E;F;G;H;I;J"
	variable idex, numWaves=10

	if (HR_TimeParamSeriesOpen)
		SpecType="Open"
	elseif(HR_TimeParamSeriesClosed)
		SpecType="Closed"
	else	
		SpecType="Diff"
	endif
	if (HR_TimeParamPikaSum)
		dataType="ResidSum"
	elseif(HR_TimeParamPikaBaseline)
		dataType="HRBk"
	else	
		dataType="MSS" + specType+"BaseL_p"		// these were not named ideally		
		specType = ""
	endif
	if (HR_TimeParamOrigData)
		mathType="Original"
	elseif(HR_TimeParamPercentData)
		mathType="Percent of Squirrel stick"
	endif
	
	if(stringMatch(ctrlName, "butt_PrintAllCrsrVals") )
		for(idex=0;idex<10;idex+=1)
			CsrStr = stringfromList(idex, CursorList)
			CsrInfoList  = CsrInfo($CsrStr)
			wave HRParam_trace_xx = $"root:HR_ParamPanel:"+StringByKey("TNAME",CsrInfoList)
			tempStr = "// HR Params -  Data Type: "+dataType+" Spectra Type:"+SpecType+" Math Type: "+MathType
			tempStr+= " Run # :"+ num2str(rn_series[pcsr($CsrStr)])+ " m/z:" +num2str(HR_TimeParamSeriesMzStart+idex)+ " Value: " +num2str(HRParam_trace_xx[pcsr($CsrStr)])
			print tempStr
		endfor
	else
		ControlInfo popup_ParamGraph_csr 
		CsrInfoList  = CsrInfo($S_Value)
		wave HRParam_trace_xx = $"root:HR_ParamPanel:"+StringByKey("TNAME",CsrInfoList)
		tempStr = "// HR Params -  Data Type: "+dataType+" Spectra Type:"+SpecType+" Math Type: "+MathType
		tempStr+= " Run # :" + num2str(rn_series[pcsr($S_Value)])+ " m/z:"+ num2str(HR_TimeParamSeriesMzStart+V_Value-1)+ " Value: " +num2str(HRParam_trace_xx[pcsr($S_Value)])
		print tempStr
	endif
End


Function UM_butt_ManyParam(ctrlName) : ButtonControl
	String ctrlName

	PK_SetUpParamPanel()

End

// a function for splitting up the CHO family into 2 other families:
//  CHO1 which have only one Oxygen and
// CHOgt1 which has greater than one Oxygen
//  1.04J
//Function UM_MakeHR_familyCHOx()
//
//	wave/t ExactMassText = root:HR_sticks:ExactMassText
//	wave mask_CHO = root:Int_Sticks:mask_CHO
//	wave/t HR_familyName = root:HR:HR_familyName
//	wave/t HR_familyListDefinition = root:HR:HR_familyListDefinition
//	wave HR_family_R = root:HR:HR_family_R
//	wave HR_family_G = root:HR:HR_family_G
//	wave HR_family_B = root:HR:HR_family_B
//	
//	variable idex, num, pos, len
//	string HRStr
//	num= numpnts(ExactMassText)
//	make/o/n=(num) 	root:Int_Sticks:mask_CHO1=nan, root:Int_Sticks:mask_CHOgt1=nan
//	
//	wave mask_CHO1 = root:Int_Sticks:mask_CHO1
//	wave mask_CHOgt1 = root:Int_Sticks:mask_CHOgt1
//	
//	for (idex=0;idex<num;idex+=1)
//		if (mask_CHO[idex]==1)
//			HRStr = 	ExactMassText[idex]
//			pos = strsearch(HRStr, "O", 0)
//			len = strlen(HRStr)
//			// we can have several cases - either there is a number after O or there one regular O and one isotope of O
//			if (pos == len-1)
//				mask_CHO1[idex]=1
//			elseif (grepstring(HRStr[pos+1], "[2-9]") )
//				mask_CHOgt1[idex]=1		
//			else
//				mask_CHO1[idex]=1
//			endif
//		endif
//	
//	endfor
//
//	if(sq_TextIntextWave("CHO1", HR_familyName)< 0)		// this is not already a family
//		num = numpnts(HR_familyName)
//		insertpoints num, 2, HR_familyName,HR_familyListDefinition,HR_family_R, HR_family_G, HR_family_B
//		HR_familyName[num]="CHO1"
//		HR_familyName[num+1]="CHOgt1"
//		HR_familyListDefinition[num, num+1]="default"
//		HR_family_R[num]=65280		// 1 is a pink
//		HR_family_G[num]=32768
//		HR_family_B[num]=58880
//		HR_family_R[num+1]=26112	// gt1 is a dark brown
//		HR_family_G[num+1]=17408
//		HR_family_B[num+1]=0
//	endif
//	
//	// for now we also need to move the CHO family to the bottom - Pika expects fragments to be a part of only one family
//	num = numpnts(HR_familyName)
//	pos = sq_TextIntextWave("CHO", HR_familyName)
//	if ( pos !=num-1)
//		deletepoints pos, 1, HR_familyName,HR_familyListDefinition,HR_family_R, HR_family_G, HR_family_B
//		insertpoints num, 1, HR_familyName,HR_familyListDefinition,HR_family_R, HR_family_G, HR_family_B
//		HR_familyName[num]="CHO"
//		HR_familyListDefinition[num]="default"
//		HR_family_R[num]=0		// just make these rgb values black for now
//		HR_family_G[num]=0
//		HR_family_B[num]=0
//	endif
//	
//End
//


// 1.04J,modified in 1.06H
// given any stick wave, sum to the unit mass values
Function UM_SumHRtoUMR(ExactStickWave,UMRresultWave, HRSpeciesFlag)
wave ExactStickWave, UMRresultWave
variable HRSpeciesFlag

	wave SpeciesMassWave = root:HR_frag:SpeciesMassWave
	wave/z MassStartPnt_FitTodo = root:HR_Sticks:MassStartPnt_FitTodo
	wave/z MassEndPnt_FitTodo = root:HR_Sticks:MassEndPnt_FitTodo
	wave/z UnitMass_FitTodo = root:HR_Sticks:UnitMass_FitTodo	//1.08E
	if (!WaveExists(UnitMass_FitTodo) || !WaveExists(MassStartPnt_FitTodo) || !WaveExists(MassEndPnt_FitTodo))		// for the case of someone upgrading who has not yet done a fit for a todo wave
		wave UnitMass_FitTodo = root:HR_Sticks:UnitMass	//1.08E
		wave MassStartPnt_FitTodo = root:HR_Sticks:MassStartPnt	//1.08E
		wave MassEndPnt_FitTodo = root:HR_Sticks:MassEndPnt	//1.08E
	endif

	variable firstMz = UnitMass_FitTodo[0]		// the first place we fit masses, typically 12
	variable lastMz = UnitMass_FitTodo[numpnts(UnitMass_FitTodo)-1]
	variable newRange = lastMz - firstMz+1
	
	make/o/n=(lastMz) root:HR_Sticks:amu_HR = p+1		// for use in plotting
	make/t/o/n=(lastMz) root:HR_Sticks:amu_HR_text = num2str(p+1)		// for use in plotting

	redimension/n=(lastMz) UMRresultWave
	UMRresultWave[0, firstMz-2]=0		// if the firstMz = 12,set masses 1 - 11, rows 0 - 10, to zero
	
	if (HRSpeciesFlag)	
		make/o/N=(lastMz) temp_MassStartPnt, temp_MassEndPnt
		wave MassStartPnt_FitTodo = temp_MassStartPnt
		wave MassEndPnt_FitTodo = temp_MassEndPnt
		MassStartPnt_FitTodo =binarysearch(SpeciesMassWave, firstMz+p - 0.5)+1
		MassEndPnt_FitTodo = binarysearch(SpeciesMassWave, firstMz+p + 0.5)
		MassEndPnt_FitTodo[newRange-1] = numpnts(SpeciesMassWave)-1		//1.03F
	endif

//	UMRresultWave[firstMz-1,lastMz-1] =  MassEndPnt_FitTodo[p-(firstMz-1)] >= MassStartPnt_FitTodo[p-(firstMz-1)] ? sum(ExactStickWave, MassStartPnt_FitTodo[p-(firstMz-1)],MassEndPnt_FitTodo[p-(firstMz-1)]) : 0
	UMRresultWave[firstMz-1,lastMz-1] =  MassEndPnt_fitTodo[p-(firstMz-1)] >= MassStartPnt_fitTodo[p-(firstMz-1)] ? sum(ExactStickWave, MassStartPnt_fitTodo[p-(firstMz-1)],MassEndPnt_fitTodo[p-(firstMz-1)]) : 0

	killwaves/z temp_MassStartPnt, temp_MassEndPnt

End



//1.20E
Function/s PK_RemainderFormula(massStr, atomWave, AtomAbun)
string massStr
wave/t atomWave
wave AtomAbun

	string atom, remainder=massStr 
	variable idex, num = numpnts(atomWave)
	
	for (idex=0;idex<num;idex+=1)
		if (AtomAbun[idex]==0)
			continue
		elseif(AtomAbun[idex]==1)
			remainder = replaceString(atomWave[idex] ,remainder, "")
		else
			atom = atomWave[idex] + num2istr(AtomAbun[idex])
			remainder = replaceString(atom ,remainder, "")
		endif
	endfor
	
	return remainder

End

