#pragma TextEncoding = "Windows-1252"
#pragma rtGlobals=3		// Use modern global access method.
//#pragma version=1.17d// decided to not use pragma but a string constant 

//PK frag Functions for High ToF-AMS analysis Toolkit
// Peak Integration by Key Analysis (PIKA)
//Copyright (C) 2005, 2006, 2007,2008, 2009,2010,2011, 2012, 2013, 2014, 2015, 2016, 2017
// Pete DeCarlo, Allison Aiken, Jose Jimenez, Donna Sueper University of Colorado
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
//
///////////////////////////////////////////////////////////////////////////////
// CURRENT STATUS
// see HR read me file for details.
 
 
//new in 1.06C
// For each species in the OnlySpecList, and using the root:HR_frag:HR_frag_* text waves, generate 
// squirrel translatable root:frag_HRair frag text waves.
// called from PK_MS_concentration
Function HR_generateFinalFrags([OnlySpecList, OnlySpecFragList, partialSpeciesList, partialFragList])		//1.10P
string OnlySpecList, OnlySpecFragList, partialSpeciesList, partialFragList

	wave/t ExactMassText_All = root:HR:ExactMassText_All		// we want to find any masses not fit but used in the HR frag table
	wave/d ExactMassWave_All = root:HR:ExactMassWave_All

	wave/t/z ExactMassText_FitTodo = root:HR_sticks:ExactMassText_FitTodo						// 1.08E replaced with *_FitTodo
	wave/t/z ExactMassFamilyText_FitTodo = root:HR_sticks:ExactMassFamilyText_FitTodo		//1.06G  // 1.08E replaced with *_FitTodo
	wave/z ExactMassWave_FitTodo = root:HR_sticks:ExactMassWave_FitTodo					// 1.08E replaced with *_FitTodo
	if (!WaveExists(ExactMassText_FitTodo) || !WaveExists(ExactMassWave_FitTodo) | !WaveExists(ExactMassFamilyText_FitTodo))
		wave/t ExactMassText_FitTodo = root:HR_sticks:ExactMassText			// 1.08E replaced with *_FitTodo
		wave ExactMassWave_FitTodo = root:HR_sticks:ExactMassWave		// 1.08E replaced with *_FitTodo
		wave/t ExactMassFamilyText_FitTodo = root:HR_sticks:ExactMassFamilyText		// 1.08E replaced with *_FitTodo
	endif

	wave/t  HR_specMassAlgebra = root:HR_frag:HR_specMassAlgebra // wave of HR species that we do math on - these may or may not be fit with Pika
	wave/t HR_specFrag_List = root:HR_frag:HR_specFrag_List
	wave/t HR_spec_List = root:HR_frag:HR_spec_List		// contains the abbreviated version of HR species such as HROrg, HRNH4, etc

	svar HRMassesNotFitList = root:HR_frag:HRMassesNotFitList		// list of HR species that are in HR_specMassAlgebra but were not fit
	svar HRMassesNotFitPosList = root:HR_frag:HRMassesNotFitPosList 	// reverse-sort list of positions in the exactMassWave_fitTodo where the HRMassesNotFitList are inserted
	svar HRMassesNotFitPosListinAll = root:HR_frag:HRMassesNotFitPosListinAll	// reverse sort list of positions in the ExactMass_ALL where HR mass exists.

	nvar HR_ShowAlerts = root:HR_panel:HR_ShowAlerts		//1.10Q
	variable idex, idexNF, numNF, pos, numSpecies, posAll, numHRfragMasses, jdex, num, numFrag
	string specRoot, tempList, fragRoot		//1.10C

	// we don't want to try to interpret a frag entry that has only a space
	gen_removeSpacesTextWaves("root:HR_frag", "HR_frag_")		// added in 1.47G

	HR_GetListNewHRAlgebraMass()		 // sets HRMassesNotFit* strings, does HR ion sanity check
	if (strlen(HRMassesNotFitList)>0)
		print "// The list of HR ions not fit using pika but used in the HR frag table is "+HRMassesNotFitList
	endif
	
	numHRfragMasses = numpnts(HR_specMassAlgebra)
	numNF = itemsInList(HRMassesNotFitList)	// number of masses not fit, but to add
	
	duplicate/o ExactMassText_FitTodo, root:HR_frag:SpeciesMassText	// will have masses fit and other masses  // 1.08E replaced with *_FitTodo
	duplicate/o ExactMassWave_FitTodo, root:HR_frag:SpeciesMassWave									// 1.08E replaced with *_FitTodo
	duplicate/o ExactMassFamilyText_FitTodo, root:HR_frag:SpeciesFamilyText								// 1.08E replaced with *_FitTodo
	wave/t  SpeciesMassText = root:HR_frag:SpeciesMassText
	wave SpeciesMassWave = root:HR_frag:SpeciesMassWave
	wave/t SpeciesFamilyText = root:HR_frag:SpeciesFamilyText		

	// add new mass in HR frag table that were not fit to the SpeciesMassText and SpeciesMassWaves
	for (idexNF=0;idexNF<numNF;idexNF+=1)
		pos = str2num(stringfromlist(idexNF,HRMassesNotFitPosList))
		posAll = str2num(stringfromlist(idexNF,HRMassesNotFitPosListinAll))
		insertpoints pos, 1, SpeciesMassText, SpeciesMassWave, SpeciesFamilyText
		SpeciesMassWave[pos]=ExactMassWave_All[posAll]
		SpeciesMassText[pos]=ExactMassText_All[posAll]
		SpeciesFamilyText[pos]=""
	endfor

	if (ParamIsDefault(OnlySpecFragList))		// do all frag waves because OnlySpecList is NOT specified
		numSpecies = numpnts(HR_specFrag_List)
	else		// do only selected frags
		numSpecies = ItemsInList(OnlySpecFragList)
		// we declare HR_spec_List and HR_specFrag_List to be new, temporary waves
		make/o/n=1/t/FREE TempSpecListWave, TempFragListWave
		gen_ListToWave(TempSpecListWave, OnlySpecList)
		wave/t HR_spec_List = TempSpecListWave
		gen_ListToWave(TempFragListWave, OnlySpecFragList)
		wave/t HR_specFrag_List = TempFragListWave
	endif
			
  	// before we create the HR frag waves, lets update our family definitions
	UM_GenerateAxByMaskWaves(ExactMassText_FitTodo, "_FitTodo")
	
	//create temporary SpecMask and add new masses.  We generate a temp_"+specRoot+"_Mask" wave for each species.  So far this only has the family designations.
	for (idex=0;idex<numSpecies;idex+=1)
		specRoot = HR_spec_List[idex]		// i.e. HROrg or HROrg43 or HRNH4
		pos=sq_findStr(root:HR_frag:HR_spec_list,specRoot,1)		// IMPORTANT that we use root:HR_frag:HR_spec_list	
		specRoot = specRoot[2, strlen(specRoot)-1]	// NO3 or Org.  Remove first 2 characters	
		HR_createTempSpeciesMask(specRoot, pos)		// uses the family designations to enter 1s and nans
		wave temp_SpecMask = $"root:HR_frag:temp_"+specRoot+"_Mask"	// created by HR_createTempSpeciesMask above
		for (idexNF=0;idexNF<numNF;idexNF+=1)		// now go through all the HR ions that were not fit but exist in the HR frag table
			pos = str2num(stringfromlist(idexNF,HRMassesNotFitPosList))
			insertpoints pos, 1, temp_SpecMask
			temp_SpecMask[pos]=nan
		endfor
	endfor
	// at this point we have our first estimate of a species, which is the waves root:HR_frag:temp_"+specRoot+"_Mask"	 which have 1s and 0s for family designations.

	// prepare to change frag entries for HR ions like C2H3O to numeric values so that we can use the squirrel-like code for deciphering frag entries
	make/o/n=(numpnts(root:HR_Frag:SpeciesMassWave)) root:HR_frag:HR_fragIonIndex=p+1

	NewDataFolder/o root:HR_ms_mats
	if (ParamIsDefault(OnlySpecFragList) )
		HR_convertFragInstances()		// goes from HR_frag_air to frag_HRair  The frag_HRair waves are just like the squirrel frag waves.  No more HR ions like C2H3O in the frag entries
	else
		HR_convertFragInstances(OnlyFragList=OnlySpecFragList)
	endif
	svar HRMassesNotFitButInHRFrag = root:HR_frag:HRMassesNotFitButInHRFrag
	if (HR_ShowAlerts && strlen(HRMassesNotFitButInHRFrag)>0)
		abort "These HR ions exist in an HR frag wave but were not fit and are not in the HR_specMassAlgebra wave in the HR frag table: "+HRMassesNotFitButInHRFrag[0,20] + "Remove all instances in all HR frag waves"
	endif
	
	// combine the specMaskWaves (root:HR_frag:air_Mask) with the converted frag waves, root:HR_frag:temp_frag_air to get root:HR_frag:frag_HRAir
	// for each species
	for (idex=0;idex<numSpecies;idex+=1)	// note that we only go through all the HR_frag waves.  Maybe instead we should go through all the species via the batch table?
		specRoot = HR_spec_List[idex]		// i.e. HROrg or HROrg43 or HRNH4
		specRoot = specRoot[2, strlen(specRoot)-1]		// i.e. Org43 or NO3	
		wave temp_SpecMask = $"root:HR_frag:temp_"+specRoot+"_Mask"	// created in HR_createTempSpeciesMask
		fragRoot = HR_specFrag_List[idex]		//1.10C to help distinguish between ammonium and NH4 in batch table 
		fragRoot = stringFromList(0, replacestring("HR_frag_", fragRoot, ""), "[")
		wave/t/z temp_frag_spec = $"root:HR_frag:temp_frag_"+fragRoot		// created in HR_convertFragInstances		//1.10P
		if (WaveExists(temp_frag_spec))
			make/t/o/n=(numpnts(temp_SpecMask)) $"root:HR_frag:frag_HR"+specRoot
			wave/t frag_HRspec = $"root:HR_frag:frag_HR"+specRoot		// will have the 'final', squirrel-translatable frag wave
			frag_HRspec =selectString( temp_SpecMask[p]==1, "", num2str(p+1))		// we need p+1 because squirrel m/z starts at 1
			
			// for each element in the HR_specMassAlgebra wave
			for (jdex=0;jdex<numHRfragMasses;jdex+=1)		// do the substitution for all entries in HR_specMassAlgebra
				pos = sq_findStr(SpeciesMassText,HR_specMassAlgebra[jdex],1)  //  find place in big SpeciesMassText where HR ion lives
				if (pos>=0)
	//				print temp_frag_spec[jdex], ReplaceString("HR_frag_",temp_frag_spec[jdex] , "frag_HR")
					frag_HRspec[pos]=temp_frag_spec[jdex]  // set to HR frag value
					frag_HRspec[pos] = ReplaceString("HR_frag_", frag_HRspec[pos] , "frag_HR")
				else
					abort "There was something wrong with element "+HR_specMassAlgebra[jdex]+"   aborting from HR_generateFinalFrags"
				endif
			endfor
		endif
//		killwaves/z temp_SpecMask, temp_frag_spec	// 1.10C
	endfor

	if (ParamIsDefault(partialFragList) )
		Abort "Maybe this shouldn't happen."
		// pk_ConvertPartialFrags()		//1.09E
	else
		PK_ConvertPartialFrags(partialSpeciesList, partialFragList)
	endif
		
	// clean up
	SetDataFolder root:HR_frag
	tempList = WaveList("temp_*", ";", "")		// "true" HR species. air, ammonium, chloride, nitrate, organic, pco2, Pwater
	sq_KillWavesInList(tempList)
	SetDataFolder root:
	
	//  this next section seems quite clunky
	HR_SpecShortLongName()		//1.10C
	wave/t HRSpecShortName = root:HR_frag:HRSpecShortName
	wave/t HRSpecLongName = root:HR_frag:HRSpecLongName
	num = numpnts(HRSpecShortName)
	
	SetDataFolder root:HR_frag:
	tempList = WaveList("frag_*", ";", "")		// "true" HR species. air, ammonium, chloride, nitrate, organic, pco2, Pwater
	SetDataFolder root:
	numFrag = itemsInLIst(tempList)
	for (idex=0;idex<numFrag;idex+=1)
		wave/t frag = $"root:HR_frag:"+stringFromList(idex, tempList)
		for (jdex=0;jdex<num;jdex+=1)
			frag = ReplaceString("frag_HR"+HRSpecLongName[jdex], frag[p], "frag_HR"+HRSpecShortName[jdex])
		endfor
	endfor	
	
End


//new in 1.06C
// for every wave root:HR_frag:HR_frag_species, make a squirrel-translatable temp_frag_species wave by replacing {HRspecis} by a number
// Also sets HRMassesNotFitButInHRFrag
Function HR_convertFragInstances([OnlyFragList])
string OnlyFragList

	// SpeciesMass waves are created/modified in HR_generateFinalFrags
	wave/t SpeciesMassText = root:HR_frag:SpeciesMassText
	wave SpeciesMassWave = root:HR_frag:SpeciesMassWave

	svar HRMassesNotFitButInHRFrag = root:HR_frag:HRMassesNotFitButInHRFrag

	variable idex, num, jdex, pos1, pos2, numList, posAll, len, index
	string specRoot, fragList, fragStr, fragEntry, massStr

	if (ParamIsDefault(OnlyFragList))
		SetDataFolder root:HR_frag
		fragList = WaveList("HR_frag_*", ";", "")
		SetDataFolder root:
	else
		fragList = OnlyFragList
	endif
	
	numList = ItemsInLIst(fragList)		// currently we perform this for all HRspecies
	
	HRMassesNotFitButInHRFrag=""
	for (idex=0;idex<numList;idex+=1)	// go through every HR frag wave
		fragStr =stringfromlist(idex,fragList) 		// i.e. HR_frag_air
		wave/t HR_frag = $"root:HR_frag:"+fragStr
		num = numpnts(HR_frag)
		duplicate/o HR_frag, $"root:HR_frag:"+replacestring("HR_frag", fragStr, "temp_frag")	// i.e. temporary wave frag_air
		wave/t frag = $"root:HR_frag:"+replacestring("HR_frag", fragStr, "temp_frag")	
		for (jdex=0;jdex<num;jdex+=1)		// go through every row in HR_frag wave
			fragEntry = HR_frag[jdex]		// i.e. {N},-HR_frag_nitrate[{N}]
			len=strlen(fragEntry)
//			print fragEntry // testing
			if (len>0)		// frag entry is NOT blank
				pos1=0
				pos2=0
				do	// check to find any sets of {}
					pos1 = strsearch(fragEntry, "{", pos2)
					pos2 = strsearch(fragEntry, "}", pos1  )
//					print pos1, pos2 // testing
					if (pos1>=0 && pos2>pos1 && pos2<len)		// we have {something} within the frag entry
						massStr = fragEntry[pos1+1, pos2-1]
//						print massStr	// testing
						index = sq_findStr(SpeciesMassText, massStr, 1)
						if (index>=0)		// we replace {N} with 34		// 34 is some number corresponding to where N is in list
							fragEntry = ReplaceString("{"+massStr+"}", fragEntry, num2str(index+1)) 	// we need to +1 to be similar to squirrel m/z indexing
						else	// cannot find massStr in SpeciesMassText
							fragEntry = ReplaceString("{"+massStr+"}", fragEntry, "0") 
							HRMassesNotFitButInHRFrag+=massStr+";"
						endif
					endif
					pos1=pos2
				while(pos2<len-1 && pos2>=0 && pos1>=0)
				frag[jdex]=fragEntry
			endif
		endfor
	endfor

//	if (strlen(missingHRIons)>0)
//		abort "Could not find mass "+missingHRIons+" in SpeciesMassText!   Aborting from HR_convertFragInstances"
//	endif

End


//new in 1.06C  // modified in 1.10C
 //  makes $"root:HR_frag:temp_"+specRoot+"_Mask" that has only 1s and nans for family inclusion
Function HR_createTempSpeciesMask(specStr, idex)
string specStr
variable idex

	wave/t HR_specFamilyBase = root:HR_frag:HR_specFamilyBase
	wave/t HR_specFamilyExceptions = root:HR_frag:HR_specFamilyExceptions

	// get the list of HR ions that were fit for this todo
	wave/t/z ExactMassText_FitTodo = root:HR_sticks:ExactMassText_FitTodo			// 1.08E replaced with *_FitTodo
	if (!WaveExists(ExactMassText_FitTodo))
		wave/t ExactMassText_FitTodo = root:HR_sticks:ExactMassText			// 1.08E replaced with *_FitTodo
	endif

	variable num, pos, kdex
	string specFamilyList, familyStr, FamilyExceptionsStr, fragWaveStr
		
	// prep the species mask wave
	make/o/n=(numpnts(ExactMassText_FitTodo) ) $"root:HR_frag:temp_"+specStr+"_Mask"=0				// 1.08E replaced with *_FitTodo
	wave speciesMask = $"root:HR_frag:temp_"+specStr+"_Mask"
	
	// if there is a family base, do these family mask waves exist?
	specFamilyList = HR_specFamilyBase[idex]
	specFamilyList = ReplaceString(" ", specFamilyList, "") 		// remove any extra spaces
	specFamilyList = ReplaceString(",", specFamilyList, ";") 		// replace  comma with semicolon
	num = itemsInList(specFamilyList)
	for(kdex=0; kdex< num;kdex+=1)
		familyStr = ReplaceString("family", stringfromList(kdex, specFamilyList)	, "")		// go from familyCH to CH	
		wave/z mask_spec = $"root:Int_sticks:mask_"+familyStr+"_fitTodo"		//1.08E  // should be updated before this function is called
		if (  !WaveExists(mask_spec) )
			wave/z mask_spec = $"root:Int_sticks:mask_"+familyStr		//1.08E
			if (  !WaveExists(mask_spec) )
				HR_butt_HRbatchTable("")
				Abort "Had problems finding the wave root:Int_sticks:mask_"+familyStr	+" for the species "+specStr+".  Is there a typo?"  //1.21B
			endif
		endif
		speciesMask  = speciesMask || mask_spec		// the union
	endfor

	FamilyExceptionsStr = HR_specFamilyExceptions[idex]
	if (strsearch(FamilyExceptionsStr, "[", 0)>=0)
		FamilyExceptionsStr = ReplaceString(" ", FamilyExceptionsStr, "")
		string myTemp =stringFromList(0,FamilyExceptionsStr, "-") 
		variable minMz = str2num(mytemp[1, strlen(mytemp)-1])
		myTemp =stringFromList(1,FamilyExceptionsStr, "-") 
		variable maxMz = str2num(mytemp[0, strlen(mytemp)-2])
		if ( ! (minMz<maxMz))
			abort "Could not understand the entry "+HR_specFamilyExceptions[idex]+" aborting from HR_createTempSpeciesMask"
		endif
		FamilyExceptionsStr = pk_MakeListOfHRionsInRange(minMz, maxMz)
//		print FamilyExceptionsStr
	endif
	
	num = ItemsInList(FamilyExceptionsStr)
	for(kdex=0; kdex< num;kdex+=1)
		pos = sq_findStr(ExactMassText_FitTodo,stringfromList(kdex, FamilyExceptionsStr),1)			// 1.08E replaced with *_FitTodo
		if (pos<0)
			print "// Could not find the mass exception "+stringfromList(kdex, FamilyExceptionsStr)+" Ignoring this exception for species HR"+specStr
		else
			speciesMask[pos] = 0  // nan
		endif
	endfor	
		
End


//1.11
Function/s pk_MakeListOfHRionsInRange(minMz, maxMz)
variable minMz, MaxMz

	string returnStr=""
	variable idex, num, pos1, pos2

 	// get the correct waves for the currently selected todo wave
 	PK_WavesForHRResults()	
	wave/t exactMassText_FitTodo	= root:HR_sticks:ExactMassText_FitTodo
	wave exactMassWave_FitTodo = root:HR_sticks:exactMassWave_FitTodo
	
 	minMz = round(minMz)
	maxMz = round(maxMz)
	
	num=numpnts(exactMassWave_FitTodo)
	make/i/Free/n=(num) integerMz
	integerMz = round(exactMassWave_FitTodo)
	
	FindLevel/q/p/edge=1 integerMz, minMz		// edge=1 increasing
	pos1 = V_flag ? 0 : V_levelX

	FindLevel/q/p/edge=1 integerMz, maxMz+1	// edge=1 increasing
	pos2 = V_flag ? num-1 : V_levelX-1 

	if (pos1> pos2)
		abort "The min and max mz values  of "+ num2str(minMz)+" "+num2str(Maxmz)  +" don't make sense.  Aborting from pk_MakeListOfHRionsInRange"	
	endif
	for (idex=pos1; idex<=pos2; idex+=1)
		returnstr+=exactMassText_FitTodo[idex]+";"
	endfor
	
	return returnStr

End

//new in 1.06C
// Now a kinda tricky part.  Look to see what masses in HR algebra masses table were *not* fit in pika.  Now add these masses to our final list.
// This function relies on HR_specMassAlgebra being in increasing order.
// sets up the important string variables
//  HRMassesNotFitList 		
//  HRMassesNotFitPosList
//  HRMassesNotFitPosListinAll
Function HR_getListNewHRAlgebraMass()

	wave/t ExactMassText_All = root:HR:ExactMassText_All
	wave/d ExactMassWave_All = root:HR:ExactMassWave_All
	wave/T HR_specMassAlgebra = root:HR_frag:HR_specMassAlgebra

	wave/t/z ExactMassText_FitTodo = root:HR_sticks:ExactMassText_FitTodo			// 1.08E replaced with *_FitTodo
	wave/z ExactMassWave_FitTodo = root:HR_sticks:ExactMassWave_FitTodo		// 1.08E replaced with *_FitTodo
	if (!WaveExists(ExactMassText_FitTodo) || !WaveExists(ExactMassWave_FitTodo))
		wave/t ExactMassText_FitTodo = root:HR_sticks:ExactMassText			// 1.08E replaced with *_FitTodo
		wave ExactMassWave_FitTodo = root:HR_sticks:ExactMassWave		// 1.08E replaced with *_FitTodo
	endif
		
	svar HRMassesNotFitList = root:HR_frag:HRMassesNotFitList
	svar HRMassesNotFitPosList = root:HR_frag:HRMassesNotFitPosList
	svar HRMassesNotFitPosListinAll = root:HR_frag:HRMassesNotFitPosListinAll

	variable pos, posAll, idex, num, massVal, newPos
	string oneStr
	
	HR_sanitySpecMassAlgebra()
	
	num = numpnts(HR_specMassAlgebra)
	HRMassesNotFitList=""
	HRMassesNotFitPosList=""
	HRMassesNotFitPosListinAll=""

	// we go from highest mass to smallest mass
	for(idex=num-1; idex>=0;idex-=1)
		oneStr = HR_specMassAlgebra[idex]
		pos = sq_findStr(ExactMassText_FitTodo,oneStr,1)				// 1.08E replaced with *_FitTodo
		if (pos<0)		// not fitted!  we need to add
			posAll = sq_findStr(ExactMassText_All, oneStr,1)		// where it exists in ExactMassText_all
			// pos should be >=0 from the HR_sanitySpecMassAlgebra()
			massVal = ExactMassWave_All[posAll]
			newPos = BinarySearch(ExactMassWave_FitTodo, massVal) + 1
			HRMassesNotFitList +=oneStr+";"		
			HRMassesNotFitPosList+=num2Str(newPos)+";"
			HRMassesNotFitPosListinAll+=num2Str(posAll)+";"
		endif
	endfor	

End


//new in 1.06C
// make sure that the masses in HR_specMassAlgebra exist in the master list of all masses and that the entries are in increasing mass order.
// also check to make sure that all HR_frag waves are the same length
Function HR_sanitySpecMassAlgebra()

	wave/t HR_specMassAlgebra = root:HR_frag:HR_specMassAlgebra
	wave/t ExactMassText_All = root:HR:ExactMassText_All
	wave/d ExactMassWave_All = root:HR:ExactMassWave_All
	
	variable idex, num, pos, numList
	string oneStr, fragList
	
	num = numpnts(HR_specMassAlgebra)
	
	// first check for existence
	make/o/n=(num) tempMass, temp
	
	for (idex=0;idex<num;idex+=1)
		oneStr = HR_specMassAlgebra[idex]
		pos = sq_findStr(ExactMassText_All,oneStr,1)
		if (pos<0)	// does not exist in master list
			abort "Could not find HR ion "+oneStr+" in the table of all HR ions (All HR ions table). HR ion names are case and order sensitive.  Aborting from  Sanity_HR_specMassAlgebra"
		else
			tempMass[idex] = ExactMassWave_All[pos]
		endif
	endfor
	
	// check for increasing m/z order, duplication
	temp = tempMass
	sort tempMass, tempMass
	temp -=tempMass
	wavestats/q/m=1 temp
	if (V_min!=0 || V_max !=0)
		abort "The HR ions in HR_specMassAlgebra are not in increasing m/z order.  Please change the order of HR ions to be increasing.  Aborting from  Sanity_HR_specMassAlgebra"
	endif
	temp[1,] = tempMass[p] - tempMass[p-1]
	wavestats/q/m=1/r=[1, num-1] temp
	if (V_min==0 || V_max ==0)
		abort "There is a duplicate entry in HR_specMassAlgebra.  Aborting from Sanity_HR_specMassAlgebra"
	endif
	temp[0,numpnts(temp)-2] = tempMass[p] - tempMass[p+1]
	wavestats/q/m=1/r=[0, num-2] temp
	if (V_min==0 || V_max ==0)
		abort "There is a duplicate entry in HR_specMassAlgebra.  Aborting from Sanity_HR_specMassAlgebra"
	endif
	
	SetDataFolder root:HR_frag
	fragList = WaveList("HR_frag_*", ";", "")
	SetDataFolder root:
	
	//check that all HR frag waves are the same length
	numList = ItemsInLIst(fragList)
	for (idex=0;idex<numList;idex+=1)
		wave frag = $"root:HR_frag:"+stringFromList(idex, fragList, ";")
		if (numpnts(frag)!=num)	// does not exist in master list
			abort "All the HR_frag waves in root:HR_frag: need to have the same number of points.   There's a problem with "+NameofWave(frag) +" Aborting from  Sanity_HR_specMassAlgebra"
		endif
	endfor
	
	killWaves/z tempMass, temp

End

    
//new in 1.06C
//Evaluates a frag list item, and assigns it to a mz bin
// JDA fix for 1.41, added an exclude parameter
Function /t HR_fragList_pass(itemStr,length,excludeStr)
	string itemStr,excludeStr
	variable length
	
	wave /t HR_spec_list=root:HR_frag:HR_spec_list
	wave /t HR_specfrag_list=root:HR_frag:HR_specFrag_list
	wave HR_specCalfac_list=root:HR_frag:HR_specCalfac_list

	string prefix,dest,subItemStr
	variable num1, num2, kw, jdex, mz, kdex, flag,b1,b2,mz1,mz2,bailout=0

	mz=0
	dest=""
	flag=0

	num1=itemsinlist(itemStr,"*")
	if (num1>1)
		prefix=removelistitem(num1-1,itemStr,"*")
		itemStr=stringfromlist(num1-1,itemStr,"*")
	else
		prefix=""
	endif
	if (stringmatch(itemStr[0],"-"))
		prefix+="-1*"
		itemStr[0,0]=""
	endif

	num1=itemsinlist(prefix,"*")
	//& Bailout if a zero is found
	for (jdex=0;jdex<num1;jdex+=1)
		if (str2num(stringfromlist(jdex,prefix,"*"))==0)
			bailout=1
			break
		endif
	endfor	
	
	//& Bailout if the item contains the exclude string
	if (strlen(excludeStr)>0 && strsearch(itemStr,excludeStr,0,2)>-1)
		bailout=1
	endif

	if (bailout)
		flag=1
		dest=""
	else
		if (stringmatch(upperstr(itemStr),"ALL"))		//& The 'all' keyword
			itemStr=num2str(MZ_START_VALUE)+"-"+num2str(length)		//	itemStr=num2str(length)
		endif
		
		num1=strlen(itemStr)
		kw=0
		for(jdex=0;jdex<num1;jdex+=1)		//& Search for illegal characters
			if (strsearch(FRAG_CHARS,itemStr[jdex],0)==-1)
				kw=1
				break
			endif
		endfor
		
// Alot of this code was simply copied over from squirrel.  
// Changes to help make things more friendly to HR ions have been added in 1.09E
		if (kw==1)
			if (stringmatch(upperstr(itemStr[0,6]),"FRAG_HR"))		//slight change from SQ
				b1=strsearch(itemStr,"[",0)
				b2=strsearch(itemStr,"]",0)
				if (b1>5 && b2>b1+1)
					wave /t/z fragwave=root:HR_frag:$(itemStr[0,b1-1])  //slight change from SQ
					if (waveexists(fragwave))
						subItemStr=itemStr[b1+1,b2-1]
						if (itemsinlist(subitemStr,"-")==1)
							mz=str2num(subItemStr)
//							if (numtype(mz)!=0)	// it is an HR ion like CO2  // 1.09E			endif
							if (mz<=numpnts(fragwave))
								num1=itemsinlist(fragwave[mz-1],",")
								dest=""
								for (jdex=0;jdex<num1;jdex+=1)
									dest+=prefix+stringfromlist(jdex,fragwave[mz-1],",")+","
								endfor
							endif
						else
							mz1=str2num(stringfromlist(0,subItemStr,"-"))
							mz2=str2num(stringfromlist(1,subItemStr,"-"))
							if (mz1>0 && mz2>=mz1)
								for (jdex=mz1;jdex<=mz2;jdex+=1)
									dest+=prefix+itemStr[0,b1]+num2str(jdex+1)+"],"
								endfor
							endif
						endif
					endif
				else
					wave /t/z fragwave=root:HR_frag:$itemStr		//slight change from SQ
					if (waveexists(fragwave))
						num1=numpnts(fragwave)
						for (jdex=0;jdex<num1;jdex+=1)
							if (strlen(fragwave[jdex])>0)
								dest+=prefix+itemStr+"["+num2str(jdex+1)+"],"
							endif
						endfor
					endif
				endif
			else			//& Reference to other species
				if (stringmatch(itemStr[0],"$"))
					kw=2		// use RIE
					itemStr[0,0]=""
				endif
				num1=numpnts(HR_spec_list)		//1.17G bug was spec_list
				for (jdex=0;jdex<num1;jdex+=1)
					if (stringmatch(upperstr(itemStr),upperstr(HR_spec_list[jdex])))		//slight change from SQ
						num2=itemsinlist(HR_specFrag_list[jdex],",")		//slight change from SQ
						for (kdex=0;kdex<num2;kdex+=1)
							if (kw==2)    // use RIE
 								dest+=prefix+num2str(HR_specCalfac_list[jdex])+"*"+stringfromlist(kdex,HR_specFrag_list[jdex],",")+","		//slight change from SQ
							else   		// don't user RIE
								dest+=prefix+stringfromlist(kdex,HR_specFrag_list[jdex],",")+","	//slight change from SQ
							endif
						endfor
						break
					endif
				endfor
			endif
			
		elseif (itemsinlist(itemStr,"-")>1)		//& Range of m/z's
			mz1=str2num(stringfromlist(0,itemStr,"-"))
			mz2=str2num(stringfromlist(1,itemStr,"-"))
			if (mz1<=mz2)
				for (jdex=mz1;jdex<=mz2;jdex+=1)
					dest+=prefix+num2str(jdex)+","
				endfor
			endif
		
		else		//& Legal entry
			mz1=str2num(itemStr)
			if (numtype(mz1)==0)
				mz=mz1
				dest=prefix+num2str(mz1)+","
				flag=1
			endif
		endif
	endif		// if bailout
	
	return num2str(mz)+";"+dest+";"+num2str(flag)

End


//new in 1.06C
// Evaluates all keywords in a fragment list
// Note that the skeleton of this function is the same as sq_fragList_convert
// The biggest difference is in the use of HR_frag_pass instead of sq_frag_pass.  Perhaps the
// HR_FragList_convert and the sq_fraglist_convert will merge.
Function HR_fragList_convert(fragList,destWaveStr,length,excludeStr)
	string fragList, destWaveStr,excludeStr	
	variable length
	
	variable idex,numInterimList,flag,mz, jdex, simpleRecursionCount, simpleRecursionMax, maxUserDefined, userDefIndex, UserDefPos
	string interimList,dest,passStr, UserDefStr 
	
	simpleRecursionMax=10^3		// arbitrary
	
	make /o/t/n=(length) $destWaveStr
	wave /t destWav=$destWaveStr
	destWav=""

	interimList=replacestring(" ",fragList,"")
	
	string AssociateStr=""
	
	// first do all the easy ones, the ones that just have a number (mass stick) in the frag entry
	do
		dest=""
		numInterimList=itemsinlist(interimList,",")
		for (idex=0;idex<numInterimList;idex+=1)
			passStr=HR_fraglist_pass(stringfromlist(idex,interimList,","),length,excludeStr)
			AssociateStr = HR_collapseHRfragSyntaxToList(passStr, AssociateStr)		
			mz=str2num(stringfromlist(0,passStr))
			if (mz==0)
				dest+=stringfromlist(1,passStr)
			elseif (mz<=length+maxUserDefined)		// 1.10K modification
				destwav[mz-1]+=stringfromlist(1,passStr)
			endif
		endfor
		interimList=dest
	while (strlen(dest)>0)
	
	// next take care of any entries that have algebra
	for (jdex=0;jdex<length;jdex+=1)
		interimList=destwav[jdex]
		if (strlen(interimList)>0)
			simpleRecursionCount=0
			do
				simpleRecursionCount+=1
				dest=""
				flag=1
				numInterimList=itemsinlist(interimList,",")
				for (idex=0;idex<numInterimList;idex+=1)
					passStr=HR_fraglist_pass(stringfromlist(idex,interimList,","),length,excludeStr)

					mz=str2num(stringfromlist(0,passStr))
					if (mz<=length+maxUserDefined)		// 1.10K modification
						dest+=stringfromlist(1,passStr)
						flag*=str2num(stringfromlist(2,passStr))
					endif
				endfor
				interimList=dest
			while (flag==0 && simpleRecursionCount<simpleRecursionMax)
			if (simpleRecursionCount<simpleRecursionMax)
				destWav[jdex]=interimList
			else
				print "// Recursion found for "+fragList+"!!"	
				break		
			endif
		endif
	endfor

//	print AssociateStr

End



//new in 1.06C
// squirrel version 1.47 way.  We no longer create 2 d matricies, but instead user sparse matricies.
Function HR_frag_matrix(fragwave,destname,method) //& 1.43D removed frag_excludestr
	wave /t fragwave
	string destname//, frag_excludeStr // 1.43D
	variable method		// note that the method parameter is no longer used beginning in 1.47
	
	variable idex, jdex, kdex, num1, num2, v_facitem
	variable numFragWave, fact, mz
	string listItemStr,facitem,facStr,depStr, tempStr, sparseStr
	
	numFragWave=numpnts(fragWave)
	make /o/n=(numFragWave,numFragWave) fragMatrix_tmp
	
	fragMatrix_tmp=0
	depStr=""
	sparseStr=""
	
	for (idex=0;idex<numFragWave;idex+=1)		// for every m/z entry
		
		num1=itemsinlist(fragWave[idex],",")		// number of , separated items in frag wave row
		for (jdex=0;jdex<num1;jdex+=1)		// for every factor in one m/z entry, such as  air[14] has 2 factors, 14,-frag_nitrate[14]
			// we build up the matrix fm_tmp by adding factors
			listItemStr=stringfromlist(jdex,fragWave[idex],",")	// get the next , separated item in fragWave[idex] 
			num2=itemsinlist(listItemStr,"*")		// number of * separated items
			mz=str2num(stringfromlist(num2-1,listItemStr,"*"))	// the last value in list separated by *
			if (numtype(mz)==0)
				fact=1		// the value that will eventually go in the fm_tmp wave
				facstr=""
				// go through all values separated by *
				for (kdex=0;kdex<num2-1;kdex+=1)
					facitem=stringfromlist(kdex,listItemStr,"*")
					if (stringmatch(facitem[0],"-"))		//^ not clear why we need to fuss with this... why not just let the str2num handle negatives? //& It's so you can put negatives in front of time-dependent operators
						fact*=-1
						facitem[0,0]=""
					endif
					v_facitem=str2num(facitem)
					if (!numtype(v_facitem))
						fact*=v_facitem
					else
						facStr+=facitem+"*"
					endif
				endfor
				if (strlen(facstr)==0)
					fragMatrix_tmp[idex][mz-1]+=fact
					sparsestr+=num2str(mz-1)+","+num2str(idex)+","+num2str(fact)+";"		//1.46A
				else
					depstr+=num2str(mz-1)+","+num2str(idex)+","+facstr+num2str(fact)+";"
				endif
			endif	// end if the last list item is a real mz number
			
		endfor		// end for every comma separated entry in FragWave[idex]
		
	endfor		// end for every row, idex, in FragWave

	matrixop /o $(destname+"_mat")=fragMatrix_tmp^t		// makes the species_mat wave
	imagetransform sumAllCols fragMatrix_tmp
	wave w_sumcols
	duplicate /o W_sumcols,$(destname+"_vec")	// makes the species_vec wave

//1.46A
// Generate sparse matrix operator
	num1=itemsinlist(sparsestr)
	make /o/n=(num1,3) $(destname+"_sparse")
	wave sparse=$(destname+"_sparse")
	for (idex=0;idex<num1;idex+=1)
		listitemstr=stringfromlist(idex,sparsestr)
		sparse[idex][]=str2num(stringfromlist(q,listitemstr,","))
	endfor
	
//1.46A
// Generate time dependency operator
	num1=itemsinlist(depstr)
	if (num1>0)
		make /o/n=(num1,2) $(destname+"_dep")
		make /o/n=(num1)/t $(destname+"_depstr")
		wave wdep=$(destname+"_dep")
		wave /t wdepstr=$(destname+"_depstr")
		for (idex=0;idex<num1;idex+=1)
			listitemstr=stringfromlist(idex,depstr)
			wdep[idex][]=str2num(stringfromlist(q,listitemstr,","))
			wdepstr[idex]=stringfromlist(2,listitemstr,",")
		endfor
	else
		killwaves /z $(destname+"_dep"),$(destname+"_depstr")
	endif
	
	// clean up
	killwaves /z fragMatrix_tmp, fragWave_tmp, fragWave_Key, fragMatrix_tmpNeg, fragWave_NegKey w_sumcols

End


//new in 1.06C
// function to extract frag info for all species in a spec list
// returns param with these key values:	fraglist:rie_list:ce_list:speclist:spec_corr:
// has to be a bit different - we need to know if we want to return only HR species, or one for families and HR masses, or both
// HRresultsType==1= return only HR species
// HRresultsType==2 = return only non HR species
// HRresultsType==3 = HR species and non HR species
Function HR_getParamStr(param,speclist, msflag,HRresultsType)
	string &param,&speclist
	variable msflag, HRresultsType
	
//	wave/t ExactMassText=root:HR_sticks:ExactMassText		//1.08E
	wave/t HR_familyName = root:HR:HR_familyName

	wave/z/t HR_specFrag_list=root:HR_frag:HR_specFrag_list
	wave/z/t HR_spec_list=root:HR_frag:HR_spec_list
	wave/z/t HR_specname_list=root:HR_frag:HR_specname_list
	wave/z HR_specIEfac_list=root:HR_frag:HR_specIEfac_list
	wave/z HR_specCEfac_list=root:HR_frag:HR_specCEfac_list
	wave/z/t HR_specCorr_list=root:HR_frag:HR_specCorr_list
	wave/z/t HR_CEWave_List=root:HR_frag:HR_CEWave_List		//1.10Q

	variable idex, numInList, loc
	string fragStr, RIEStr, CEStr, SpecStr, HRspecStr, specCorrStr, specItem, CECorrStr

	if (!WaveExists(HR_specFrag_list) || !WaveExists(HR_specCorr_list))		// only check the first and last waves
		abort "The HR frag waves do not exist in the HR_frag folder. Aborting from HR_getParamStr"
	endif
	
	if (!WaveExists(HR_CEWave_List) )		// only check the first and last waves
		make/t/o/n=(numpnts(HR_specFrag_list)) root:HR_frag:HR_CEWave_List/wave=HR_CEWave_List
		HR_CEWave_List=""
	endif
	
	fragStr="fraglist:"
	RIEStr="rie_list:"
	CEStr="ce_list:"
	specStr="speclist:"
	specCorrStr="spec_corr:"
	CECorrStr="CE_corr:"
	speclist = ReplaceString(" ", speclist, "") // remove any extra spaces
	numInList=itemsinlist(speclist, ",")
	
	for(idex=0;idex<numInList;idex+=1)
		specItem=stringfromlist(idex,speclist,",")
		loc=sq_findstr(HR_spec_list,specItem,1)
		if(loc==-1)		
			loc=sq_findstr(HR_specname_list,specItem,1)
		endif
		if (loc==-1)	// not a species.  Could be an individual HR mass or a family or all
			if(  stringmatch(lowerStr(specitem), "family*")  && sq_findstr(HR_familyName,specItem,0) && exists("root:int_sticks:mask_"+replacestring("family", specitem, "") )  )
				specStr+=specItem+"/"
				fragStr+=specItem+"/"
				RIEStr+="1/"
				CEStr+="1/"			
				specCorrStr+="/"
			elseif(sq_findstr(HR_specFrag_list,specItem,1) )  //  specItem is an HR ion like C3H2O
				specStr+=specItem+"/"
				fragStr+="1/"					//1.08E fix
				RIEStr+="1/"
				CEStr+="1/"
				specCorrStr+="/"
			elseif (cmpstr(specItem,"all")==0)
				specStr+="all/"
				fragStr+="all/"
				RIEStr+="1/"
				CEStr+="1/"
				specCorrStr+="/"
			endif
		else	 // i.e. HRair, HROrg or HROrg44
			specStr+=specItem+"/"		// modified in 1.10C
			fragStr+="frag_"+specItem+"/" // ReplaceString("HR_frag_",HR_specFrag_list[loc], "frag_HR")+"/"		// 1.10C change
			if(msflag)
				RIEStr+="1/"
				CEStr+="1/"
				specCorrStr+="/"
				// we do not include species correction waves in this, because the species correction wave is time-series dependent.
			else
				RIEStr+=num2str(HR_specIEfac_list[loc])+"/"
				CEStr+=num2str(HR_specCEfac_list[loc])+"/"
				specCorrStr+=HR_specCorr_list[loc]+"/"
				CECorrStr+=HR_CEWave_List[loc]+"/"
//				RIEStr+=num2str(IEfac_list[loc])+"/"
//				CEStr+=num2str(CEfac_list[loc])+"/"
//				waves2PlotFacList+=num2str(IEfac_list[loc]*CEfac_list[loc])+"/"
//				specCorrStr+=speccorr_list[loc]+"/"
			endif

		endif
	endfor
	param+=specStr+";"+fragStr+";"+RIEStr+";"+CEStr+";"+specCorrStr+";"+CECorrStr+";"
	
End



// Tables similar to the squirrel batch and frag tables
//new in 1.06C
Function HR_butt_HRfragTable(ctrlName) : ButtonControl
	String ctrlName

	DoWIndow/F HRFragTable
	if (V_flag==1)
		return 0
	endif

	Edit/N=HRFragTable/W=(90,200,700,400)  as "HRFragTable"
	
	HR_populate_HRFragTable()
End

Function  HR_populate_HRFragTable()

	if (!DataFolderExists("root:HR_frag"))
		abort "You must import the HR frag waves into a HR_frag data folder first.  Aborting from  HR_batchTable"
	endif
	
	variable idex, num
	
	wave/t HR_specFrag_list = root:HR_frag:HR_specFrag_list
	wave HR_specColor_R = root:HR_frag:HR_specColor_R
	wave HR_specColor_G = root:HR_frag:HR_specColor_G
	wave HR_specColor_B = root:HR_frag:HR_specColor_B
	wave HR_SpecMassAlgebra = root:HR_frag:HR_SpecMassAlgebra
	
	AppendToTable root:HR_frag:HR_SpecMassAlgebra
	// we need for this table to have all the HR_frag waves in case the user adds an HR ion to the rows of Hr ions to do algebra on.
	SetDataFolder root:HR_frag:
	String AllFragWaveList = WaveList("HR_frag_*", ";", "" )
	SetDataFolder root:
	num = numpnts(root:HR_frag:HR_specFrag_list)
	String fldrSav0= GetDataFolder(1)
	SetDataFolder root:HR_frag:
	for (idex=0;idex<num;idex+=1)		//1.07A
		wave/t/z fragWave = $"root:HR_frag:"+HR_specFrag_list[idex]		//1.09G
		if (WaveExists(fragWave))
			AppendToTable fragWave
			ModifyTable alignment($NameOfWave(fragWave))=0,width($NameOfWave(fragWave))=120,style($NameOfWave(fragWave))=1
			ModifyTable rgb($NameOfWave(fragWave))=(HR_specColor_R[idex],HR_specColor_G[idex],HR_specColor_B[idex])
			AllFragWaveList = RemoveFromList(HR_specFrag_list[idex], AllFragWaveList)
		endif
	endfor
	num=itemsInList(AllFragWaveList)
	for (idex=0;idex<num;idex+=1)		//1.07A
		wave/t/z fragWave = $"root:HR_frag:"+StringFromList(idex, AllFragWaveList)		//1.09G
//		if (WaveExists(fragWave))
			AppendToTable fragWave
//			ModifyTable alignment($NameOfWave(fragWave))=0,width($NameOfWave(fragWave))=120,style($NameOfWave(fragWave))=1
//			ModifyTable rgb($NameOfWave(fragWave))=(HR_specColor_R[idex],HR_specColor_G[idex],HR_specColor_B[idex])
//		endif
	endfor
		
	ModifyTable format(Point)=1,alignment(HR_SpecMassAlgebra)=0,width(HR_SpecMassAlgebra)=69
	SetDataFolder fldrSav0
End

//new in 1.06C
Function HR_butt_HRbatchTable(ctrlName) : ButtonControl
	String ctrlName

	DoWIndow/F HRBatchTable
	if (V_flag==1)
		return 0
	endif
	if (!DataFolderExists("root:HR_frag"))
		abort "You must import the HR frag waves into a HR_frag data folder first.  Aborting from  HR_batchTable"
	endif
	wave/z/t HR_CEWave_List = root:HR_frag:HR_CEWave_List
	if (!WaveExists(HR_CEWave_List))
		make/t/o/n=(numpnts(root:HR_frag:HR_specCEfac_list)) root:HR_frag:HR_CEWave_List/wave=HR_CEWave_List
		HR_CEWave_List=""
	endif
	String fldrSav0= GetDataFolder(1)
	SetDataFolder root:HR_frag:
	wave/t HR_specname_list,HR_spec_list,HR_SpecFrag_list
	wave/t HR_specFamilyBase,HR_specFamilyExceptions, HR_specIEFac_list, HR_specCalFac_list, HR_specCEfac_list
	wave/t HR_specCorr_list, HR_specColor_R, HR_specColor_G, HR_specColor_B
	Edit/N=HRBatchTable/W=(80,50,700,250) HR_specname_list,HR_spec_list,HR_SpecFrag_list as "HRBatchTable"
	AppendToTable HR_specFamilyBase,HR_specFamilyExceptions, HR_specIEFac_list, HR_specCalFac_list, HR_specCEfac_list,HR_CEWave_List
	AppendToTable HR_specCorr_list, HR_specColor_R, HR_specColor_G, HR_specColor_B
	ModifyTable format(Point)=1,width(HR_SpecFrag_list)=102,width(HR_specFamilyBase)=102
	ModifyTable width(HR_specFamilyExceptions)=102
	SetDataFolder fldrSav0
End


//There are 3 types of HR things in specList, specFlavor Returns the subset of each
// 1 -> HR mass, like C3H2O
// 2 -> HR family, like familyAir
// 4 -> HR species, like HRair
// HR_CheckListForHRSpecies(specList, 3)		 returns all valid non-HRspecies entities, single HR ions and families
// HR_CheckListForHRSpecies(specList, 4)		 returns all valid HRspecies entities
// HR_CheckListForHRSpecies(specList, 6)		 returns all valid HRspecies and familiy entities
// returns a comma separated list
Function/s HR_CheckListForHRSpecies(specList, specFlavor)
string specList
variable specFlavor

	wave/t HR_specFrag_list=root:HR_frag:HR_specFrag_list
	wave/t HR_spec_list=root:HR_frag:HR_spec_list
	wave/t HR_specname_list=root:HR_frag:HR_specname_list
	wave/t HR_familyName=root:HR:HR_familyName
	
	wave/t/z ExactMassText_FitTodo=root:HR_sticks:ExactMassText_FitTodo			// 1.08E replaced with *_FitTodo
	if (!WaveExists(ExactMassText_FitTodo) )
		wave/t ExactMassText_FitTodo = root:HR_sticks:ExactMassText		// 1.08E replaced with *_FitTodo
	endif

	variable idex, numList, loc
	string specItem, flavoredSpecList

	flavoredSpecList=""
	specList = replaceString(" ", specList, "")
	numList = itemsInList(specList, ",")
	
	for(idex=0;idex<numList;idex+=1)
		specItem=stringfromlist(idex,speclist,",")
		
		// first check for HR species
		loc=sq_findstr(HR_spec_list,specItem,1)
		if(loc==-1)		
			loc=sq_findstr(HR_specname_list,specItem,1)
		endif
		
		if (loc>=0 && specFlavor&4)
//			specItem= HR_HandleDollarSignHRFrag(specItem, loc)		// can generate i.e. CombineAtEnd:HRTotal;HRNO3/HRNH4/
			flavoredSpecList+=specItem+","
		elseif(  stringmatch(lowerStr(specitem), "family*")>=0  && sq_findstr(HR_familyName,replacestring("family", specitem, ""),0)>=0 &&specFlavor&2  )
			flavoredSpecList+=specItem+","
		elseif( sq_findstr(ExactMassText_FitTodo,specItem,1)>=0 && specFlavor&1)		//1.11B fix	// 1.08E replaced with *_FitTodo
			flavoredSpecList+=specItem+","
		elseif(stringmatch("all", speclist) && specFlavor==3)		//1.07B
			flavoredSpecList+=specItem+","
		elseif(strlen(specItem)==0)
			abort "There seems to be something wrong with the HR result list in step 6i. Is there a typo?"
		elseif(loc<0 && sq_findstr(ExactMassText_FitTodo,specItem,1)<0 && specFlavor&1)	//1.11B fix	//1.10Q
			abort "The HR ion "+specItem+ " was not fit. Cannot proceed to use it. If part of a species, createa separate entry in the batch table. Aborting from HR_CheckListForHRSpecies"
		endif
				
	endfor
	
	return flavoredSpecList

End


//**********************************************************************************************
//******************** SQ vs Pika *************

Function HR_button_sqVsPk(ctrlName) : ButtonControl
	String ctrlName

	HR_SQvsPK_calcWaves()
 	
 	doWIndow/f SqVsPk_Panel
  	if(V_flag==0)
 		HR_ViewSqVsPk_Panel()
 	endif
 	
	HR_plotSqVsPkGrs()
	
End


Function	HR_plotSqVsPkGrs()
		
	svar SQvsPKInfo=root:HR_panel:SQvsPKInfo
	svar todoStr=root:panel:sq_toDoWvNm
	nvar HRResults_dataType=root:HR_panel:HRResults_dataType // a numerical value

	string Species, specInitial

 	ControlInfo/W=HR_Analysis_Panel HRResults_pop_dataSet
	if(HRResults_dataType==4)
		specInitial="M"/// m for Open minus closed
	else 
		specInitial="D"
	endif

 	ControlInfo/W=sqVsPk_Panel tabSpecies
	Species = S_value
	
	doWIndow/f sqVsPk_Panel

	setwindow sqVsPk_Panel,userdata(todoStr)=todoStr
	setwindow sqVsPk_Panel,userdata(specInitial)=specInitial
	setwindow sqVsPk_Panel,userdata(Species)=Species

	nvar HR_checkUseAB = root:HR_panel:HR_checkUseAB
	nvar HR_checkConvertTougm3 = root:HR_panel:HR_checkConvertTougm3
	SQvsPKInfo =""
	SQvsPKInfo +="todo: "+todoStr +"\r"
	SQvsPKInfo +="units: "+SelectString(HR_checkConvertTougm3, "Hz","ug/m3")  +"\r"
	SQvsPKInfo +="use AB correction: "+SelectString(HR_checkUseAB, "No"," Yes")+"\r"
	SQvsPKInfo +="pika data type: "+SelectString(stringmatch(specInitial, "D"), "OMinusClosed","Diff") 

	SetActiveSubwindow sqVsPk_Panel#HR_SQvsPkTSGraph
	gen_removeTracesFromGraph("sqVsPk_Panel#HR_SQvsPkTSGraph")
	HR_populate_SQvsPkTSGraph()
	SetActiveSubwindow ##
	
	SetActiveSubwindow sqVsPk_Panel#HR_SQvsPkTSCorrGraph
	gen_removeTracesFromGraph("sqVsPk_Panel#HR_SQvsPkTSCorrGraph")
 	HR_populate_SQvsPkTSCorrGraph(win="sqVsPk_Panel#HR_SQvsPkTSCorrGraph")
	SetActiveSubwindow ##
	
	SetActiveSubwindow sqVsPk_Panel#HR_SQvsPkMSGraph
	gen_removeTracesFromGraph("sqVsPk_Panel#HR_SQvsPkMSGraph")
	HR_populate_SQvsPkMSGraph()
	SetActiveSubwindow ##
	
	SetActiveSubwindow sqVsPk_Panel#HR_SQvsPkMSCorrGraph
	gen_removeTracesFromGraph("sqVsPk_Panel#HR_SQvsPkMSCorrGraph")
	HR_populate_SQvsPkMSCorrGraph()
	SetActiveSubwindow ##

End


Function/s HR_SQvsPK_calcWaves()

	// flags on Pk panel
	nvar HR_checkUseAB = root:HR_panel:HR_checkUseAB
	nvar HR_checkConvertTougm3 = root:HR_panel:HR_checkConvertTougm3
	nvar HR_checkAskForSuffix=root:HR_panel:HR_checkAskForSuffix
	nvar HR_checkCalcHRErrors=root:HR_panel:HR_checkCalcHRErrors
	nvar HRResults_dataType=root:HR_panel:HRResults_dataType // a numerical value
	nvar NormalizeSpectra = root:HR_panel:NormalizeSpectra		//1.09K

	nvar HR_checkConvertNansTo0 = root:HR_panel:HR_checkConvertNansTo0
	nvar HR_checkConvertNegsTo0 = root:HR_panel:HR_checkConvertNegsTo0

	// flags on Sq panel
	nvar ms0_corr=root:panel:ms0_corr
	nvar	ms0_ion2mass=root:panel:ms0_ion2mass
	nvar ms0_msTseries_suffix_ck=root:panel:ms0_msTseries_suffix_ck
	nvar ms0_appendErrorBars=root:panel:ms0_appendErrorBars	
	nvar ms0_dataType=root:panel:ms0_dataType		// a numerical value

	svar HRResults_ionList=root:HR_panel:HRResults_ionList
	nvar HR_checkAvgHRMS_UMR = root:HR_panel:HR_checkAvgHRMS_UMR

	nvar ams_time_saved=root:panel:ams_time_saved

	svar todoStr=root:panel:sq_toDoWvNm
	wave/z todo=$(todoStr)
	
	string waveNameSuffix, spectraType, specTypeInitial, paramStrPartial, SQstr1="", PKstr2="", str3=""
	
	if (HR_checkUseAB)
		sq_ABsanity()
	endif
 	ControlInfo/W=HR_Analysis_Panel HRResults_pop_dataSet
	if (stringMatch(S_Value, "Initialize") || stringMatch(S_Value, "Get List"))
		abort "Please select the data type from the drop down menu."
	else
		spectraType=S_Value
	endif

	// check to see that the species have the same RIE and CE and specCorr and CEWave_List values
	str3="Org,NO3,SO4,NH4,Chl"		// order imp
	sq_getParamStr(SQstr1,str3,0, PKstr2)
	str3="HROrg,HRNO3,HRSO4,HRNH4,HRChl"  	// order imp
	PKstr2=""
	HR_getParamStr(PKstr2,str3,0, 1)
	if (!stringMatch(StringByKey("rie_list", SQstr1,":",  ";"), StringByKey("rie_list", PKstr2,":",  ";")) )
 		sq_batch_GuideUserToMismatch(StringByKey("rie_list", SQstr1,":",  ";"), StringByKey("rie_list", PKstr2,":",  ";"), "RIE")			// 1.15D
	endif
	if (!stringMatch(StringByKey("ce_list", SQstr1,":",  ";"), StringByKey("ce_list", PKstr2,":",  ";")) )
 		sq_batch_GuideUserToMismatch(StringByKey("ce_list", SQstr1,":",  ";"), StringByKey("ce_list", PKstr2,":",  ";"), "CE")			// 1.15D
	endif
	if (!stringMatch(StringByKey("spec_corr", SQstr1,":",  ";"), StringByKey("spec_corr", PKstr2,":",  ";")) ) 
		sq_batch_GuideUserToMismatch(StringByKey("spec_corr", SQstr1,":",  ";"), StringByKey("spec_corr", PKstr2,":",  ";"), "spec_corr")			// 1.15D
	endif
	if (!stringMatch(StringByKey("CE_corr", SQstr1,":",  ";"), StringByKey("CE_corr", PKstr2,":",  ";")) )
		if (stringmatch(StringByKey("CE_corr", SQstr1,":",  ";"),"CE_fphase/CE_fphase/CE_fphase/CE_fphase/CE_fphase/")  && stringmatch(StringByKey("CE_corr", PKstr2,":",  ";"),"HRCE_fphase/HRCE_fphase/HRCE_fphase/HRCE_fphase/HRCE_fphase/") )
		else	
			sq_batch_GuideUserToMismatch(StringByKey("CE_corr", SQstr1,":",  ";"), StringByKey("CE_corr", PKstr2,":",  ";"), "CE_corr")			// 1.15D
		endif
	endif
	
// prepare SQ and Pk calculations

	NormalizeSpectra=0
	
	//1.14M
	variable numHR = PK_WavesForHRResults()		//1.08G		// changed in 1.10C to now also recalculate family designations

	waveNameSuffix = todoStr

	//  can only do for PK data type = Diff or OMinusC
 	ControlInfo/W=HR_Analysis_Panel HRResults_pop_dataSet
	if (stringMatch(S_Value, "Initialize") || stringMatch(S_Value, "Open") || stringMatch(S_Value, "Closed"))
		abort "Can only do Sq vs Pika on Diff or OMinusC spectra - aborting from HR_button_sqVsPk"
	elseif(HRResults_dataType==4)
		spectraType="HROminusCStick"
		specTypeInitial="M"/// m for Open minus closed
	else 
		spectraType = "HRDiffStick"
		specTypeInitial="D"
	endif

	// set SQ data type to diff
	sq_updateMSLists()
	svar MSStickList = root:panel:MSStickList
	ms0_dataType = FindListItem("Diff",MSStickList)+1		// +1 cuz it is a drop down menu

		
	// set species list to be the same
	sq_butt_MSSpeciesDefault( "")			// set SQ species list
	HR_butt_setResultsList( "HRResults_butt_setListAerHRfams")			// set HR species list
	
	// set AB corr, ug/m3 units to be the same, use whatever Pk panel says
	ms0_ion2mass = HR_checkConvertTougm3
	ms0_corr = HR_checkUseAB
	
	// don't deal with errors, too much work for now
	ms0_appendErrorBars = 0
	HR_checkCalcHRErrors=0
		
	// we deal with wave names ourselves
	HR_checkAskForSuffix=0			
	ms0_msTseries_suffix_ck=1		//slightly different interface in SQ

	HR_checkAvgHRMS_UMR=1		// always do avg mass spec PK in UMR
	
	ams_time_saved=1				// always do TS for Pk and SQ in as saved time units

	paramStrPartial=""
	paramStrPartial+="nanTo0_flag:"+num2str(HR_checkConvertNansTo0)+";"
	paramStrPartial+="negTo0_flag:"+num2str(HR_checkConvertNegsTo0)+";"  //bug! forgot the :  1.09D fix
	paramStrPartial+="corr_op:"+num2str(HR_checkUseAB)+";"
	paramStrPartial+="ug_op:"+num2str(HR_checkConvertTougm3)+";"	
	paramStrPartial+="t_base:;"	
	paramStrPartial+="timeweight_op:3;"		// BUg fixed in 1.09E,  previously this parameter wasn't added causing it to average over delta time between runs.	
	
	//1.14M
	paramStrPartial+="min_cl:0;max_cl:"+num2str(numHR-1)+";"

// do the calcs for the TS
	sq_MSCalc_Tseries(waveNameSuffix, 0)
	HR_calc_MSVsTime(waveNameSuffix, spectraType, specTypeInitial, paramStrPartial, "")

// do the calcs for the MS
	sq_MSCalc_avgStick(waveNameSuffix) 
	HR_calc_MSAvg(waveNameSuffix, spectraType, specTypeInitial, paramStrPartial, "")

	return specTypeInitial
	
End


//1.15D
Function sq_batch_GuideUserToMismatch(SQ_list, Pk_list, colStr)
string SQ_list, Pk_list, colStr

	string str, specList, colList
	variable idex, num, rowPosSq, rowPosPk, colPosSq, colPosPk, pos

	SQ_list=ReplaceString("/", SQ_list, ";")  	//1.19E
	Pk_list=ReplaceString("/", Pk_list, ";")
	
	specList="Org;NO3;SO4;NH4;Chl"		// order imp
	colList="RIE;CE;spec_corr;CE_corr"		// order imp
	make/free/o sqcol={3, 5, 6, 7}   // "RIE;CE;spec_corr;CE_corr"		// order imp
	make/free/o pkcol={5, 7, 8, 9}   // "RIE;CE;spec_corr;CE_corr"		// order imp
	for (idex=0;idex<itemsInList(SQ_list); idex+=1)
		if (!stringMatch(stringFromList(idex,SQ_list), stringFromList(idex,Pk_list) ) )
			Str = stringFromList(idex,specList )
			break
		endif
	endfor

	if (idex==5)
 		abort "There seems to be something wrong with these lists which should be identical:"+SQ_list + ","+pk_list+".  aborting from sq_batch_GuideUserToMismatch "
	endif
	
	pos=max(0,WhichListItem(colStr,colList ))
	
	wave/t spec_list = root:frag:spec_list
	wave/t HR_spec_list = root:HR_frag:HR_spec_list
	
	sq_butt_reviewBatch("")
	DoUpdate
	FindValue/TXOP = 4/Text=str spec_list
	rowPosSq = max(0,V_value)
	colPosSq = sqcol[pos]
	ModifyTable/W=ReviewBatchTable selection=(rowPosSq , colPosSq , rowPosSq , colPosSq , rowPosSq , colPosSq ), topLeftCell=(rowPosSq , 0 )
	
	HR_butt_HRbatchTable("")			//1.15D
	DoUpdate
	FindValue/TXOP = 4/Text=("HR"+str) HR_spec_list
	rowPosPk = max(0,V_value)
	colPosPk = pkcol[pos]
	ModifyTable/W=HRBatchTable selection=(rowPosPk , colPosPk , rowPosPk , colPosPk , rowPosPk , colPosPk ), topLeftCell=(rowPosPk , 0 )

 	abort "There are two different valuesof "+stringFromList(idex,SQ_list)+" and "+stringFromList(idex,Pk_list)+" between Squirrel and Pika in: "+str+". Aborting from sq_batch_GuideUserToMismatch"		//1.19E

End


Function HR_ViewSqVsPk_Panel()

	NewPanel/N=SqVsPk_Panel /W=(50,200,550,645)/N=sqVsPk_Panel as "SqVsPk_Panel"
	ModifyPanel fixedSize=1		//1.57L
	SetDrawLayer UserBack
	DrawText 10,90,"Time Series"
	DrawText 280,90,"Time Series Corr."
	DrawText 10,270,"Mass Spec"
	DrawText 280,270,"Mass Spec Corr."

	SetDrawEnv fstyle= 1,fsize= 18
	DrawText 31,22,"Squirrel vs Pika"

	TabControl tabSpecies,pos={5,45},size={490,390},tabLabel(0)="Org",tabLabel(1)="SO4"
	TabControl tabSpecies,tabLabel(2)="NO3",tabLabel(3)="NH4",tabLabel(4)="Chl"
	TabControl tabSpecies proc=SqVsPkTabProc

	Button HR_but_HR_SQvsPkTSGraph,pos={200,70},size={50,20},title="Pop", proc=sq_butt_popThis
	Button HR_but_HR_SQvsPkTSCorrGraph,pos={430,70},size={50,20},title="Pop", proc=sq_butt_popThis
	Button HR_but_HR_SQvsPkMSGraph,pos={200,250},size={50,20},title="Pop", proc=sq_butt_popThis
	Button HR_but_HR_SQvsPkMSCorrGraph,pos={430,250},size={50,20},title="Pop", proc=sq_butt_popThis

	TitleBox SqVsPika variable=root:HR_panel:SQvsPKInfo,frame=0, pos={350,2}
	svar SQvsPKInfo = root:HR_panel:SQvsPKInfo
	SQvsPKInfo=""

	Display/W=(10,100,270,240)/HOST=# 
	RenameWindow #,HR_SQvsPkTSGraph
	SetActiveSubwindow ##
	
	Display/W=(10,280,270,420)/HOST=# 
	RenameWindow #,HR_SQvsPkMSGraph
	SetActiveSubwindow ##
	
	Display/W=(280,100,480,240)/HOST=# 
	RenameWindow #,HR_SQvsPkTSCorrGraph
	SetActiveSubwindow ##
	
	Display/W=(280,280,480,420)/HOST=# 
	RenameWindow #,HR_SQvsPkMSCorrGraph
	SetActiveSubwindow ##

End


// mass spec correlation
Function HR_populate_SQvsPkMSCorrGraph()

	string waveNameSuffix, Species, specInitial
	
	waveNameSuffix= getuserdata("sqVsPk_Panel","","todoStr")
	specInitial = getuserdata("sqVsPk_Panel","","specInitial")
	Species = getuserdata("sqVsPk_Panel","","Species")

	wave sqSpecStick = $"root:MSSD_"+waveNameSuffix+"_"+Species
	wave HRspecStick = $"root:UMR"+specInitial+"_"+waveNameSuffix+"_HR"+Species
	wave amu_HR = root:HR_Sticks:amu_HR	// for use in plotting
	wave/t amu_HR_text = root:HR_Sticks:amu_HR_text 		// for use in plotting

	wave One2one = root:HR_panel:One2one		//1.07A
	string SpecColorList
	variable minTemp, maxTemp, maxMz

	SpecColorList = PK_getColorListSpecies(Species)

	if (WhichListItem("one2one", TraceNameList(WinName(1, 1), ";", 1 ))<0 )		//1.09H
		AppendToGraph one2one vs one2one
		ModifyGraph rgb($NameofWave(one2one))=(0,0,0)
	endif
	AppendToGraph sqSpecStick[0,numpnts(HRspecStick)] vs HRspecStick
	
	ModifyGraph mode($NameofWave(sqSpecStick))=3
	ModifyGraph msize($NameofWave(sqSpecStick))=5
	ModifyGraph mrkThick($NameofWave(sqSpecStick))=2
	ModifyGraph textMarker($NameofWave(sqSpecStick))={amu_HR_text,"default",0,0,5,0.00,0.00}
	
	ModifyGraph rgb($NameofWave(sqSpecStick))=(str2num(stringFromList(0, SpecColorList)) ,str2num(stringFromList(1, SpecColorList)),str2num(stringFromList(2, SpecColorList)))

	ModifyGraph grid=2
	ModifyGraph gridRGB=(34816,34816,34816)
	ModifyGraph nticks=9
	ModifyGraph minor=1

	Label left "Sq "+Species		// +", g/m3"
	Label bottom "Pika "+Species		// +", g/m3"

	waveStats/q sqSpecStick
	minTemp = V_min
	maxTemp = V_max 
	waveStats/q HRspecStick
	minTemp = min( V_min, minTemp)
	maxTemp = max(V_max, maxTemp)
	SetAxis left minTemp, maxTemp
	SetAxis bottom minTemp, maxTemp

End


//  t series correlation
Function HR_populate_SQvsPkTSCorrGraph([win])
string win

	string waveNameSuffix, Species, specInitial
	
	waveNameSuffix= getuserdata("sqVsPk_Panel","","todoStr")
	specInitial = getuserdata("sqVsPk_Panel","","specInitial")
	Species = getuserdata("sqVsPk_Panel","","Species")

	wave sqSpecTS =$"root:"+Species+"_"+waveNameSuffix
	wave HRspecTS = $"root:HR"+Species+"_"+specInitial+"_"+waveNameSuffix
	wave One2one = root:HR_panel:One2one		//1.07A
	
	variable minTemp, maxTemp
	string SpecColorList
	SpecColorList = PK_getColorListSpecies(Species)
 	if (ParamIsDefault(win))
 		win=""
 	endif

	if (WhichListItem("one2one", TraceNameList(win, ";", 1 ))<0 )
		AppendToGraph one2one vs one2one
		ModifyGraph rgb($NameOfWave(one2one))=(0,0,0)
	endif
	AppendToGraph sqSpecTS vs HRspecTS
	
	ModifyGraph mode($NameofWave(sqSpecTS))=2
	ModifyGraph lSize($NameofWave(sqSpecTS))=2
	ModifyGraph rgb($NameofWave(sqSpecTS))=(str2num(stringFromList(0, SpecColorList)) ,str2num(stringFromList(1, SpecColorList)),str2num(stringFromList(2, SpecColorList)))
	
	ModifyGraph grid=2
	ModifyGraph gridRGB=(34816,34816,34816)
	ModifyGraph nticks=9
	ModifyGraph minor=1
	Label left "Sq "+Species		// +", g/m3"
	Label bottom "Pika "+Species		// +", g/m3"

	waveStats/q sqSpecTS
	minTemp = V_min
	maxTemp = V_max 
	waveStats/q HRspecTS
	minTemp = min(0, min( V_min, minTemp))
	maxTemp = max(V_max, maxTemp)
	SetAxis left minTemp, maxTemp
	SetAxis bottom minTemp, maxTemp

End


// t series 
Function HR_populate_SQvsPkTSGraph()

	string waveNameSuffix, Species, specInitial
	string SpecColorList
	
	waveNameSuffix= getuserdata("sqVsPk_Panel","","todoStr")
	specInitial = getuserdata("sqVsPk_Panel","","specInitial")
	Species = getuserdata("sqVsPk_Panel","","Species")

	wave/z sqSpecTS =$"root:"+Species+"_"+waveNameSuffix
	wave/z HRspecTS = $"root:HR"+Species+"_"+specInitial+"_"+waveNameSuffix
	wave t_series = root:index:t_series
	
	//1.09A
	if (!WaveExists(sqSpecTS) || !WaveExists(HRspecTS) )
		abort "Perhaps you have changed your todo wave.  Could not find waves "+nameofWave(sqSpecTS) +" or "+nameofWave(HRspecTS) +"  aborting from HR_populate_SQvsPkTSGraph"
	endif
	
	SpecColorList = PK_getColorListSpecies(Species)

	AppendToGraph sqSpecTS vs root:index:t_series
	AppendToGraph HRspecTS vs root:index:t_series
	
	ModifyGraph lSize($NameofWave(sqSpecTS))=2
	ModifyGraph rgb($NameOfWave(HRspecTS))=(0,0,0)
	ModifyGraph rgb($NameofWave(sqSpecTS))=(str2num(stringFromList(0, SpecColorList)) ,str2num(stringFromList(1, SpecColorList)),str2num(stringFromList(2, SpecColorList)))
	
	SetAxis/A/E=3 left		//1.52O
	ModifyGraph gaps=0
	ModifyGraph grid=2
	ModifyGraph gridRGB=(34816,34816,34816)
	ModifyGraph zero(left)=1
	ModifyGraph nticks=9
	ModifyGraph minor=1

	ModifyGraph dateInfo(bottom)={0,1,0}
	Label bottom " "
	
End


// mass spec 
Function HR_populate_SQvsPkMSGraph()
	
	string waveNameSuffix, Species, specInitial
	
	waveNameSuffix= getuserdata("sqVsPk_Panel","","todoStr")
	specInitial = getuserdata("sqVsPk_Panel","","specInitial")
	Species = getuserdata("sqVsPk_Panel","","Species")

	wave sqSpecStick = $"root:MSSD_"+waveNameSuffix+"_"+Species
	wave HRspecStick = $"root:UMR"+specInitial+"_"+waveNameSuffix+"_HR"+Species
	wave amus = root:diagnostics:amus
	wave amu_HR = root:HR_Sticks:amu_HR	// for use in plotting
	wave/t amu_HR_text = root:HR_Sticks:amu_HR_text 		// for use in plotting
	
	string SpecColorList
	SpecColorList = PK_getColorListSpecies(Species)

	AppendToGraph sqSpecStick vs amu_HR
	AppendToGraph HRspecStick vs amus
	
	ModifyGraph mode=1
	ModifyGraph lSize=2
	ModifyGraph rgb($NameOfWave(HRspecStick))=(0,0,0)
	ModifyGraph offset($NameOfWave(HRspecStick))={0.3,0}
	ModifyGraph rgb($NameOfWave(sqSpecStick))=(str2num(stringFromList(0, SpecColorList)) ,str2num(stringFromList(1, SpecColorList)),str2num(stringFromList(2, SpecColorList)))

	SetAxis/A/E=3 left		//1.19A
	ModifyGraph gaps=0
	ModifyGraph grid=2
	ModifyGraph gridRGB=(34816,34816,34816)
	ModifyGraph zero(left)=1
	ModifyGraph nticks=9
	ModifyGraph minor=1

	Label left "Nitrate equivalent"
	Label bottom "m/z, HR is offset by 0.3"
	
	SetAxis bottom 1, numpnts(HRspecStick) + 1

End


Function/s PK_getColorListSpecies(Species)
string Species

	wave col_r=root:frag:col_r
	wave col_g=root:frag:col_g
	wave col_b=root:frag:col_b
	wave /z/t col_names=root:frag:col_names

	variable idex, num
	string returnList = "0;0;0"		// default to black
	
	idex=sq_findstr(col_names,Species, 1)
	
	if (idex>=0)
		return num2str(col_r[idex])+";"+num2str(col_g[idex])+";"+num2str(col_b[idex])
	endif
	
	return returnList

End


Function SqVsPkTabProc(ctrlName,tabNum) : TabControl
	String ctrlName
	Variable tabNum

 	ControlInfo/W=sqVsPk_Panel tabSpecies
	setwindow sqVsPk_Panel,userdata(Species)=S_Value

	HR_plotSqVsPkGrs()

End


// want to create frag_HRtest from "HR_frag_organic[CO2]"
// want to create frag_HRtest1 from "HR_frag_organic[44]"
// called from within HR_generateFinalFrags(), 		//1.09E
Function PK_ConvertPartialFrags(OnlyTheseSpeciesList, OnlyThesePartialList)
string OnlyTheseSpeciesList, OnlyThesePartialList

	wave/t SpeciesMassText = root:HR_frag:SpeciesMassText  // should already be set
	wave SpeciesMassWave = root:HR_frag:SpeciesMassWave // should already be set

	string speciesStr, str, str1, str2, newListFragEntry, newListSpecies
	variable idex, num, pos, pos1, pos2
	
	wave/t HR_spec_list = root:HR_frag:HR_spec_list				// waves in HR batchTable
	wave/t HR_specFrag_list = root:HR_frag:HR_specFrag_list		// waves in HR batchTable	
	
	num = itemsInList(OnlyThesePartialList)
	for (idex=0;idex<num;idex+=1)
		speciesStr=stringFromList(idex, OnlyTheseSpeciesList)  
		str=stringFromList(idex, OnlyThesePartialList)  
		splitstring/E=("HR_frag_"+"([[:alnum:]]+)\\[([[:alnum:]]+)\\]") str,str1, str2 
		wave/z/t thisFrag= $"root:HR_frag:HR_frag_"+str1			// the frag wave the user sees
		if (strlen(str1)>0 && strlen(str2)>0)
			pos = sq_TextIntextWave("HR_frag_"+str1, HR_specFrag_list)		// find the entry that has only this value... i.e. HROrg has only HR_frag_organic
			wave/z/t convertedFrag= $"root:HR_frag:frag_"+HR_spec_list[pos]	// the same frag wave converted for squirrel-like translation		// 1.10C
			if (!WaveExists(thisFrag))
				abort "There seems to be something wrong with the batch entry in row "+num2str(idex)+"  Can't parse "+str+ " aborting from pk_ConvertPartialFrags"			
			endif
			
			// find the species that corresponds to only this frag wave
			make/o/t/n=(numpnts(convertedFrag)) $"root:HR_frag:frag_"+speciesStr/Wave=FragWave
			FragWave=""
			if (str2num(str2) >=0)		// we have an m/z, such as HR_frag_organic[44]
				pos=str2num(str2)
				pos1=binarysearch(SpeciesMassWave, pos-0.5)	+1	// ie is if str2 = 44 we want to start looking at 43.5
				pos2=binarysearch(SpeciesMassWave, pos+0.5)		// ie is if str2 = 44 we want to end looking at 44.5
				FragWave[pos1,pos2]=convertedFrag[p]
			else		// we have an HR ion such as HR_frag_organic[CO2]
				pos = sq_findStr(SpeciesMassText,str2,1)			
				if (pos<0)
					abort "There seems to be something wrong with the HR ion in row "+num2str(idex)+"  Can't parse "+str+ " aborting from pk_ConvertPartialFrags"			
				endif
				FragWave[pos]=convertedFrag[p]
			endif
		endif
	endfor
	
End

//1.10C
// creates root:HR_frag:HRSpecShortName (Org etc), root:HR_frag:HRSpecLongName (organic etc)
// by going through every element in HR_spec_list
Function HR_SpecShortLongName()

	wave/t HR_spec_list = root:HR_frag:HR_spec_list
	wave/t HR_specFrag_list = root:HR_frag:HR_specFrag_list

	variable idex, num, count
	string longStr, shortStr
	
	num = numpnts(HR_spec_list)
	count =0
	make/o/t/n=0 root:HR_frag:HRSpecShortName/wave=HRSpecShortName
	make/o/t/n=0 root:HR_frag:HRSpecLongName/wave=HRSpecLongName
	
	for (idex=0;idex<num;idex+=1)
		shortStr = HR_spec_list[idex]   // HROrg
		shortStr = shortStr[2, strlen(shortStr)-1]   // Org
		longStr = HR_specFrag_list[idex]	// HR_frag_organic
		longStr = longstr[8, strlen(longstr)-1]	// organic
		if (!stringmatch(shortStr, longStr)  && strsearch(longstr, "[", 0)<0 )
			insertpoints count, 1, HRSpecShortName, HRSpecLongName
			HRSpecShortName[count] = shortStr
			HRSpecLongName[count] = longStr
			count+=1
		endif
	endfor

End


//1.20W  Tim O's function
Function HR_Species_subpanel_check(ctrlName,checked) : CheckBoxControl
	string ctrlName
	variable checked
	
	nvar/z BCdata = root:HR_frag:BCdata
	if (!NVar_exists(BCdata))
		variable/g  root:HR_frag:BCdata=0
		nvar BCdata = root:HR_frag:BCdata
	endif

	if (stringmatch(ctrlName,"*_AMS"))	// && checked)
		CheckBox HRGrps_butt_ChangeHRfrag_AMS, value=1
		CheckBox HRGrps_butt_ChangeHRfrag_SPAMS, value=0
		BCdata = 0
		HR_ck_ChangeHR4BC(ctrlName)
	elseif (stringmatch(ctrlName,"*_SPAMS"))	// && checked)
		CheckBox HRGrps_butt_ChangeHRfrag_AMS, value=0
		CheckBox HRGrps_butt_ChangeHRfrag_SPAMS, value=1
		BCdata = 1
		HR_ck_ChangeHR4BC(ctrlName)
	endif

End


//Function from Tim O 1.20W
// Rewrote Donna's function to include both switches..
Function HR_ck_ChangeHR4BC(ctrlName)  // 
	string ctrlName
// for black carbon (BC) measurements for the AMS we need to change 2 things:
// remove Cx from HROrg and make our own species with just this family
// add C to the HR frag table and divy up the C from organic to the C from the black carbon

// waves in the batch table
	wave/t HR_specName_List = root:HR_frag:HR_specName_List
	wave/t HR_spec_List = root:HR_frag:HR_spec_List
	wave/t HR_specFrag_List = root:HR_frag:HR_specFrag_List
	wave/t HR_specFamilyBase = root:HR_frag:HR_specFamilyBase
	wave/t HR_specFamilyExceptions = root:HR_frag:HR_specFamilyExceptions
	wave HR_specIEFac_list = root:HR_frag:HR_specIEFac_list
	wave HR_specCalFac_list = root:HR_frag:HR_specCalFac_list
	wave HR_specCEfac_list = root:HR_frag:HR_specCEfac_list
	wave/t HR_CEWave_List = root:HR_frag:HR_CEWave_List
	wave/t HR_specCorr_list = root:HR_frag:HR_specCorr_list
	wave HR_specColor_R = root:HR_frag:HR_specColor_R
	wave HR_specColor_G = root:HR_frag:HR_specColor_G
	wave HR_specColor_B = root:HR_frag:HR_specColor_B

//  waves in the frag table
	wave/t HR_SpecMassAlgebra  = root:HR_frag:HR_SpecMassAlgebra
	wave/t HR_frag_organic  = root:HR_frag:HR_frag_organic
	wave/t/z HR_frag_blackCarbon = root:HR_frag:HR_frag_blackCarbon	//may or may not exist
	SetDataFolder root:HR_frag:
	String AllFragWaveList = WaveList("HR_frag_*", ";", "" )
	SetDataFolder root:
	// we don't have an exhaustive list of HR frag waves because user can create others

	//Length of batch and frag waves
	variable num, numHRFragWaves, numSpeciesAlgebra
	num = numpnts(HR_specName_List)
	numSpeciesAlgebra = numpnts(HR_SpecMassAlgebra)
	numHRFragWaves = ItemsInLIst(AllFragWaveList)

	//Test batch wave lengths
	if (num!=numpnts(HR_spec_List) || num!=numpnts(HR_specFrag_List) || num!=numpnts(HR_specFamilyBase) || num!=numpnts(HR_specFamilyExceptions) || num!=numpnts(HR_specIEFac_list) || num!=numpnts(HR_specCalFac_list) )	// broken into separate lines for easier editing	
	 	if (num!=numpnts(HR_specCEfac_list) || num!=numpnts(HR_CEWave_List) ||  num!=numpnts(HR_specCorr_list) || num!=numpnts(HR_specColor_R) || num!=numpnts(HR_specColor_G) || num!=numpnts(HR_specColor_B) )
			HR_butt_HRbatchTable("")
			abort "The number of points in the different waves in the HR batch table is different - please fix before proceeding.  Aborting from HR_ck_ChangeHR4BC" 
		endif
	endif

//	Set up HRBC components
	//Batch Table:
	variable idex = sq_findStr(HR_specName_List,"HRBlackCarbon",1)
	if (idex>=0)	//HR species HRBlackCarbon already exists, so just fill in accordingly
		HR_specName_List[idex]="HRBlackCarbon"
		HR_spec_List[idex]="HRBC"
		HR_specFrag_List[idex]="HR_frag_blackCarbon"
		HR_specFamilyBase[idex]="familyCx"
		HR_specFamilyExceptions[idex]=""
		HR_specIEFac_list[idex]=0.2		// from TimO
		//	HR_specCalFac_list[idex]=5		// from TimO  should be calced automatically
		HR_specCEfac_list[idex]=1
		HR_CEWave_List[idex]=""
		HR_specCorr_list[idex]=""
		HR_specColor_R[idex]=0
		HR_specColor_G[idex]=0
		HR_specColor_B[idex]=0	
	else	//HR species HRBlackCarbon does not yet exist, so create and fill in accordingly
		insertpoints num, 1, HR_specName_List; 	HR_specName_List[num]="HRBlackCarbon"
		insertpoints num, 1, HR_spec_List; 		HR_spec_List[num]="HRBC"
		insertpoints num, 1, HR_specFrag_List; 	HR_specFrag_List[num]="HR_frag_blackCarbon"
		insertpoints num, 1, HR_specFamilyBase; 	HR_specFamilyBase[num]="familyCx"
		insertpoints num, 1, HR_specFamilyExceptions; 	HR_specFamilyExceptions[num]=""
		insertpoints num, 1, HR_specIEFac_list; 	HR_specIEFac_list[num]=0.2		// from TimO
		insertpoints num, 1, HR_specCalFac_list; //	HR_specCalFac_list[num]=5		// from TimO  should be calced automatically
		insertpoints num, 1, HR_specCEfac_list; 	HR_specCEfac_list[num]=1
		insertpoints num, 1, HR_CEWave_List; 		HR_CEWave_List[num]=""
		insertpoints num, 1, HR_specCorr_list; 	HR_specCorr_list[num]=""
		insertpoints num, 1, HR_specColor_R; 		HR_specColor_R[num]=0
		insertpoints num, 1, HR_specColor_G; 		HR_specColor_G[num]=0
		insertpoints num, 1, HR_specColor_B; 		HR_specColor_B[num]=0	
	endif

	//Frag Table:
	variable jdex=sq_findStr(HR_SpecMassAlgebra,"C",1)
	variable kdex=sq_findStr(HR_SpecMassAlgebra,"j13C",1)	

	if (jdex>=0 && waveexists(HR_frag_blackcarbon)) //HR ion C already exists in HR frag table and HR_frag_blackcarbon wave exists, so just fill in accordingly
		HR_SpecMassAlgebra[jdex]="C"
		HR_SpecMassAlgebra[kdex]="j13C"	
		HR_frag_blackCarbon=""
		HR_frag_blackCarbon[jdex]="0.625*HR_frag_blackcarbon[{C3}]"
		HR_frag_blackCarbon[kdex]="0.0108157*HR_frag_blackcarbon[{C}]"
	elseif (jdex<0 && kdex<0) // && !waveexists(HR_frag_blackcarbon))	//HR ion C does not yet exist in HR frag table and HR_frag_blackcarbon wave does not exist, so create and fill in accordingly
		insertpoints 0, 2, HR_SpecMassAlgebra; 
		HR_SpecMassAlgebra[0]="C"		// we will assume that C will be the first one in the list
		HR_SpecMassAlgebra[1]="j13C"
		variable i
		for(i=0;i<numHRFragWaves;i+=1)
			wave/t myHRFrag = $"root:HR_Frag:"+ stringFromList(i,AllFragWaveList)
			insertpoints 0, 2, myHRFrag	
			myHRFrag[0]=""
			myHRFrag[1]=""
		endfor
		make/t/o/n=(numSpeciesAlgebra+2)  root:HR_frag:HR_frag_blackCarbon/wave = HR_frag_blackCarbon
		HR_frag_blackCarbon=""
		HR_frag_blackCarbon[0]="0.625*HR_frag_blackcarbon[{C3}]"
		HR_frag_blackCarbon[1]="0.0108157*HR_frag_blackcarbon[{C}]"
	endif

//	Switch HROrg components based on AMS vs SPAMS sampling mode
	jdex=sq_findStr(HR_SpecMassAlgebra,"C",1)
	kdex=sq_findStr(HR_SpecMassAlgebra,"j13C",1)	
	variable ldex = sq_findStr(HR_specName_List,"HRorganic",1)
	string familyEntry = HR_specFamilyBase[ldex]
	
	if (stringmatch(ctrlName,"*_AMS"))
		//Batch Table:
		if (WhichListItem("familyCx",familyEntry)<0)
			HR_specFamilyBase[ldex] = AddListItem("familyCx", familyEntry)
		endif
		//Frag Table:
		HR_frag_organic[jdex] = ""	
		HR_frag_organic[kdex] = ""
	elseif (stringmatch(ctrlName,"*_SPAMS"))
		//Batch Table:
		HR_specFamilyBase[ldex] = RemoveFromList("familyCx", familyEntry)
		//Frag Table:
		HR_frag_organic[jdex] = "{C},-HR_frag_blackcarbon[{C}]"	
		HR_frag_organic[kdex] = "0.0108157*HR_frag_organic[{C}]"
	endif

	sq_make_rBC_UMR_fragNbatchWaves(nan) 		//1.11L

End


// No longer used in 1.20W because of Time O's functionality
// Tables similar to the squirrel batch and frag tables
//Function HR_butt_ChangeHR4BC(ctrlName) : ButtonControl
//string ctrlName
//// for black carbon (BC) measurements for the AMS we need to change 2 things:
//// remove Cx from HROrg and make our own species with just this family
//// add C to the HR frag table and divy up the C from organic to the C from the black carbon
//
//// waves in the batch table
//	wave/t HR_specName_List = root:HR_frag:HR_specName_List
//	wave/t HR_spec_List = root:HR_frag:HR_spec_List
//	wave/t HR_specFrag_List = root:HR_frag:HR_specFrag_List
//	wave/t HR_specFamilyBase = root:HR_frag:HR_specFamilyBase
//	wave/t HR_specFamilyExceptions = root:HR_frag:HR_specFamilyExceptions
//	wave HR_specIEFac_list = root:HR_frag:HR_specIEFac_list
//	wave HR_specCalFac_list = root:HR_frag:HR_specCalFac_list
//	wave HR_specCEfac_list = root:HR_frag:HR_specCEfac_list
//	wave/t HR_CEWave_List = root:HR_frag:HR_CEWave_List
//	
//	wave/t HR_specCorr_list = root:HR_frag:HR_specCorr_list
//	wave HR_specColor_R = root:HR_frag:HR_specColor_R
//	wave HR_specColor_G = root:HR_frag:HR_specColor_G
//	wave HR_specColor_B = root:HR_frag:HR_specColor_B
//
////  waves in the frag table
//	wave/t HR_SpecMassAlgebra  = root:HR_frag:HR_SpecMassAlgebra
//	wave/t HR_frag_organic  = root:HR_frag:HR_frag_organic
//	// we don't have an exhaustive list of HR frag waves because user can create others
//
//	variable idex, jdex, num, numHRFragWaves, numSpeciesAlgebra
//	string familyEntry
//
//	SetDataFolder root:HR_frag:
//	String AllFragWaveList = WaveList("HR_frag_*", ";", "" )
//	SetDataFolder root:
//	
//	num = numpnts(HR_specName_List)
//
//	numSpeciesAlgebra = numpnts(HR_SpecMassAlgebra)
//
//	numHRFragWaves = ItemsInLIst(AllFragWaveList)
//
//	if (num!=numpnts(HR_spec_List) || num!=numpnts(HR_specFrag_List) || num!=numpnts(HR_specFamilyBase) || num!=numpnts(HR_specFamilyExceptions) || num!=numpnts(HR_specIEFac_list) || num!=numpnts(HR_specCalFac_list) )	// broken into separate lines for easier editing	
//	 	if (num!=numpnts(HR_specCEfac_list) || num!=numpnts(HR_CEWave_List) ||  num!=numpnts(HR_specCorr_list) || num!=numpnts(HR_specColor_R) || num!=numpnts(HR_specColor_G) || num!=numpnts(HR_specColor_B) )
//			HR_butt_HRbatchTable("")
//			abort "The number of points in the different waves in the HR batch table is different - please fix before proceeding.  aborting from HR_butt_ChangeHR4BC" 
//		endif
//	endif
//
//	variable/g root:HR_frag:BCdata=1
//	// first correct the batch table
//	idex = sq_findStr(HR_specName_List,"HRBlackCarbon",1)
//	if (idex>=0)
//		HR_butt_HRbatchTable("")
//		abort "The HR species HRBlackCarbon already exists - aborting from HR_butt_ChangeHR4BC"
//	endif
//	
////  now assume we must do the insertiong
//
//	insertpoints num, 1, HR_specName_List; 	HR_specName_List[num]="HRBlackCarbon"
//	insertpoints num, 1, HR_spec_List; 		HR_spec_List[num]="HRBC"
//	insertpoints num, 1, HR_specFrag_List; 	HR_specFrag_List[num]="HR_frag_blackCarbon"
//	insertpoints num, 1, HR_specFamilyBase; 	HR_specFamilyBase[num]="familyCx"
//	insertpoints num, 1, HR_specFamilyExceptions; 	HR_specFamilyExceptions[num]=""
//	insertpoints num, 1, HR_specIEFac_list; 	HR_specIEFac_list[num]=0.2		// from TimO
//	insertpoints num, 1, HR_specCalFac_list; //	HR_specCalFac_list[num]=5		// from TimO  should be calced automatically
//	insertpoints num, 1, HR_specCEfac_list; 	HR_specCEfac_list[num]=1
//	insertpoints num, 1, HR_CEWave_List; 		HR_CEWave_List[num]=""
//
//	insertpoints num, 1, HR_specCorr_list; 		HR_specCorr_list[num]=""
//	insertpoints num, 1, HR_specColor_R; 		HR_specColor_R[num]=0
//	insertpoints num, 1, HR_specColor_G; 		HR_specColor_G[num]=0
//	insertpoints num, 1, HR_specColor_B; 		HR_specColor_B[num]=0	
//	
//	idex = sq_findStr(HR_specName_List,"HRorganic",1)			
//	familyEntry = HR_specFamilyBase[idex]			//1.12F
//	HR_specFamilyBase[idex] = RemoveFromList("familyCx", familyEntry)
//
//	jdex=sq_findStr(HR_SpecMassAlgebra,"C",1)
//	
//	if (jdex>=0)
//		HR_butt_HRFragTable("")
//		abort "The HR ion C appears to already exist in the HR frag table = aborting from HR_butt_ChangeHR4BC"
//	endif
//	
//	insertpoints 0, 2, HR_SpecMassAlgebra; 
//	HR_SpecMassAlgebra[0]="C"		// we will assume that C will be the first one in the list
//	HR_SpecMassAlgebra[1]="j13C"		// we will assume that C will be the first one in the list
//	
//	for(idex=0;idex<numHRFragWaves;idex+=1)
//		wave/t myHRFrag = $"root:HR_Frag:"+ stringFromList(idex,AllFragWaveList)
//		insertpoints 0, 2, myHRFrag	
//		myHRFrag[0]=""
//		myHRFrag[1]=""
//	endfor
//	
//	make/t/o/n=(numSpeciesAlgebra+2)  root:HR_frag:HR_frag_blackCarbon/wave = HR_frag_blackCarbon
//	HR_frag_blackCarbon=""
//	HR_frag_blackCarbon[0]="0.625*HR_frag_blackcarbon[{C3}]"
//	HR_frag_blackCarbon[1]="0.0108157*HR_frag_blackcarbon[{C}]"		// j13C
//	HR_frag_organic[0] = "{C},-HR_frag_blackcarbon[{C}]"	
//	HR_frag_organic[1] = "0.0108157*HR_frag_organic[{C}]"		// bug in 1.12E
//		
//	sq_make_rBC_UMR_fragNbatchWaves(nan) 		//1.11L
//
//End

// function below given to Donna from Tim O 4/29/2013
// modified with contributions from Leah W 5/24/2013
// 3 parts:
// (1) create new frag waves frag_lowc, frag_midc, frag_fullc
// (2) create frag_orgLessBC
// (3) take care of specific entries, i.e. mz12 at C, etc
// (4) deal with batch table
Function sq_make_rBC_UMR_fragNbatchWaves(max_Cn)    //max_Cn = maximum number of carbons; NaN/INF == use max amu in file
    variable max_Cn
    
	nvar/z BCdata = root:HR_frag:BCdata
	if (!NVar_exists(BCdata))
		variable/g  root:HR_frag:BCdata=0
		nvar BCdata = root:HR_frag:BCdata
	endif

   variable maxmz
    setdatafolder root:frag:
    wave/t frag_organic  
    wave/t frag_Cl  
    wave/t frag_HCl
    wave/z amus = root:diagnsotics:amus    //, frag_carbon
	if (!WaveExists(amus))
		maxmz=3000	// some arbitrarily large value.  //1.12E
	else
		maxmz = numpnts(amus)+1
	endif
	
	variable maxPfragOrg = dimsize(frag_organic,0)-1		//1.19D
//  step 1
    duplicate/o/t frag_organic, frag_fullc, frag_lowc, frag_midc, frag_orgLessBC
    frag_fullc = ""			// fullc means fullerenes, not 'full'
    frag_lowc = ""
    frag_midc = ""
    if (numtype(max_Cn)>0)
        max_Cn = floor(numpnts(frag_organic)/12)+1
    endif

    variable Cn
    //lowc C1 - C5
    for (Cn=1; Cn<6; Cn+=1)
        frag_lowc[Cn*12-1] = num2str(p+1)
    endfor

    //midc C6 - C29
    for (Cn=6; Cn<30; Cn+=1)
        frag_midc[Cn*12-1,Cn*12+1] = num2str(p+1)		//1.19D  presumes maxPfragOrg>=362
    endfor

    //fullerene C30-C166 (max 2000 amu)
    for (Cn=30; Cn<(max_Cn+1); Cn+=1)
//        if (Cn > 90)
//            frag_fullc[Cn*12-2,Cn*12+2] = num2str(p+1)    //try to account for problems with m/z calibration
        if (Cn*12-1 <= maxPfragOrg)			//1.19D
	        if (Cn > 49)
	            frag_fullc[Cn*12-1,min(maxPfragOrg,Cn*12+2)] = num2str(p+1)    //to account for higher j13C fractions
	        else
	            frag_fullc[Cn*12-1,min(maxPfragOrg,Cn*12+1)] = num2str(p+1)
	        endif
       endif
    endfor
    
// now take care of individual, specific C mass adjustments
	frag_lowc[11] = "0.625*frag_lowc[36]"   	; frag_orgLessBC[11] ="12,-frag_lowc[12]" 		//mz 12 = C
	frag_lowc[23] = "24,-frag_sulphate[24]"	;											//mz 24 = C2		... doubly charged SO at 48
	frag_lowc[35] = "36,-frag_HCl[36],-frag_air[36]"										//mz 36 = C3
	frag_lowc[47] = "0.101*frag_lowc[12]"	; frag_orgLessBC[47]="0.5*frag_organic[62]"		//mz 48 = C4
	frag_lowc[59] = "60,-frag_organic[60]"	; frag_orgLessBC[59]="3.32*frag_organic[74]"	//mz 60 = C5
	
	frag_midc[71] = "72,-frag_orgLessBC[72]"		;frag_orgLessBC[71]="0.37*frag_organic[73]"					//mz 72 = C6
	frag_midc[83] = "0.74*frag_midc[72]	"			;frag_orgLessBC[83]="84,-frag_sulphate[84],-frag_midc[84]"	//mz 84 = C7
	frag_midc[95] = "96,-frag_orgLessBC[96]"		;frag_orgLessBC[95]="0.49*frag_organic[97]"					//mz 96 = C8		// SO4 at 96 

	frag_midc[107] = "108,-frag_orgLessBC[108]"	;frag_orgLessBC[107]="0.5*frag_organic[94],0.5*frag_organic[122]"		//mz 108 = C9	// for frag_orgLessBC use avg of organics +-CH2 group
	frag_midc[111] = "120,-frag_orgLessBC[120]"	;frag_orgLessBC[111]="0.5*frag_organic[106],0.5*frag_organic[134]"	//mz 120 = C10 	// for frag_orgLessBC use avg of organics +-CH2 group
	frag_midc[131] = "132,-frag_orgLessBC[132]"	;frag_orgLessBC[131]="0.5*frag_organic[118],0.5*frag_organic[146]"	//mz 132 = C11 	// for frag_orgLessBC use avg of organics +-CH2 group
	frag_midc[143] = "144,-frag_orgLessBC[144]"	;frag_orgLessBC[143]="0.5*frag_organic[130],0.5*frag_organic[158]"	//mz 144 = C12 	// for frag_orgLessBC use avg of organics +-CH2 group
	frag_midc[155] = "156,-frag_orgLessBC[156]"	;frag_orgLessBC[155]="0.5*frag_organic[142],0.5*frag_organic[170]"	//mz 156 = C13 	// for frag_orgLessBC use avg of organics +-CH2 group

	frag_midc[167] = "168,-frag_orgLessBC[168]"	;frag_orgLessBC[167]="0.65*frag_organic[154]"					//mz 168 = C14
	frag_midc[179] = "180,-frag_orgLessBC[180]"	;frag_orgLessBC[179]="0.5*frag_organic[166],0.5*frag_organic[194]"	//mz 180 = C15  // for frag_orgLessBC use avg of organics +-CH2 group
	frag_midc[191] = "192,-frag_orgLessBC[192]"	;frag_orgLessBC[191]="0.53*frag_organic[178]"					//mz 192 = C16   
	frag_midc[203] = "204,-frag_orgLessBC[204]"	;frag_orgLessBC[203]="0.5*frag_organic[190],0.5*frag_organic[218]"	//mz 204 = C17 // for frag_orgLessBC use avg of organics +-CH2 group

	frag_midc[215] = "216,-frag_orgLessBC[216]"	;frag_orgLessBC[215]="0.15*frag_organic[208],0.15*frag_organic[210],0.35*frag_organic[218],0.35*frag_organic[220]"	//mz 216 = C18
	frag_midc[227] = "228,-frag_orgLessBC[228]"	;frag_orgLessBC[227]="0.13*frag_organic[218],0.13*frag_organic[220],0.37*frag_organic[230],0.37*frag_organic[232]"	//mz 228 = C19
	frag_midc[239] = "240,-frag_orgLessBC[240]"	;frag_orgLessBC[239]="0.18*frag_organic[232],0.18*frag_organic[234],0.32*frag_organic[244],0.32*frag_organic[244]"	//mz 240 = C20
	frag_midc[251] = "252,-frag_orgLessBC[252]"	;frag_orgLessBC[251]="0.14*frag_organic[244],0.14*frag_organic[244],0.36*frag_organic[254],0.36*frag_organic[256]"	//mz 252 = C21
	frag_midc[263] = "264,-frag_orgLessBC[264]"	;frag_orgLessBC[263]="0.21*frag_organic[256],0.21*frag_organic[258],0.29*frag_organic[268],0.29*frag_organic[270]"	//mz 264 = C22
	frag_midc[275] = "276,-frag_orgLessBC[276]"	;frag_orgLessBC[275]="0.23*frag_organic[268],0.23*frag_organic[270],0.27*frag_organic[282],0.27*frag_organic[282]"	//mz 276 = C23
	frag_midc[287] = "288,-frag_orgLessBC[288]"	;frag_orgLessBC[287]="0.23*frag_organic[282],0.23*frag_organic[282],0.27*frag_organic[292],0.27*frag_organic[294]"	//mz 288 = C24
	frag_midc[299] = "300,-frag_orgLessBC[300]"	;frag_orgLessBC[299]="0.21*frag_organic[292],0.21*frag_organic[294],0.29*frag_organic[304],0.29*frag_organic[306]"	//mz 300 = C25
	frag_midc[311] = "312,-frag_orgLessBC[312]"	;frag_orgLessBC[311]="0.35*frag_organic[308],0.35*frag_organic[310],0.15*frag_organic[318],0.15*frag_organic[320]"	//mz 312 = C26
	frag_midc[323] = "324,-frag_orgLessBC[324]"	;frag_orgLessBC[323]="0.29*frag_organic[318],0.29*frag_organic[320],0.21*frag_organic[330],0.21*frag_organic[332]"	//mz 324 = C27
	frag_midc[335] = "336,-frag_orgLessBC[336]"	;frag_orgLessBC[335]="0.36*frag_organic[332],0.36*frag_organic[334],0.14*frag_organic[344],0.14*frag_organic[344]"	//mz 336 = C28
	frag_midc[347] = "348,-frag_orgLessBC[348]"	;frag_orgLessBC[347]="0.32*frag_organic[344],0.32*frag_organic[344],0.18*frag_organic[354],0.18*frag_organic[356]"	//mz 348 = C29

	// now for fullerenes we consider the fact that j13C occurrs 0.0107 and C occurrs 0.9893
	variable ndex = numpnts(frag_fullc), kdex, relAbund, thisMz
	for(ndex=20;ndex<ceil(maxmz/12);ndex+=1)
		for(kdex=0;kdex<=10;kdex+=1)
			relAbund=binomial(ndex, kdex)* (0.0107/0.9893)^kdex
			if (relAbund>0.3)		// feasible grouping of 13C and C
				thisMz=12*ndex+kdex
				if (thisMz>=ndex)
					break
				endif
				frag_fullc[thisMz-1] = num2str(thisMz)				
			endif
		endfor
	endfor

	// the default abundances for Cl are 0.2424/0.7576
	if (BCdata)
		frag_Cl=""
		frag_HCl[34]="35"
		frag_HCl[35]="3.5*frag_HCl[35]"    
		frag_HCl[36]="0.323*frag_HCl[35]"
		frag_HCl[37]="0.323*frag_HCl[36]"
	else  // original UMR frag table
		frag_Cl=""
		frag_Cl[34]="35,-frag_HCl[35]"
		frag_Cl[35]="0.323*frag_Cl[35]"
		frag_HCl=""
		frag_HCl[34]="0.231*frag_HCl[36]"
		frag_HCl[35]="36,-frag_air[36]"
		frag_HCl[36]="0.323*frag_HCl[35]"
		frag_HCl[37]="0.323*frag_HCl[36]"
	endif
	
// now take care of entries in the batch table
    wave/t specname_list,spec_list,frag_list,speccorr_list
    wave/t/z CEWave_List
    if (!waveexists(CEWave_List))
        make/o/n=(numpnts(specname_list))/t CEWave_List
        wave/t CEWave_List
        CEWave_List = ""
    endif
    wave IEfac_list,calfac_list,CEfac_list
    variable np = numpnts(specname_list)
    variable i
    for (i=0;i<np;I+=1)
        if (stringmatch(specname_list[i], "rBC"))
//            Print "// Species rBC found in Batch table!  Should check and/or remove before proceeding."
//            Print "// Aborting without modifying Batch table."
            setdatafolder root:
            abort
        endif
    endfor
    insertpoints np,4,specname_list,spec_list,frag_list,IEfac_list,calfac_list,CEfac_list,CEWave_List,speccorr_list
    //lowc
    specname_list[np] = "lowc"
    spec_list[np] = "lowc"
    frag_list[np] = "frag_lowc"
    IEfac_list[np] = 0.2			// maybe this is closer to 0.4?  Message from Leah W 5/2013
    //calfac_list[np]    //calculated automatically
    CEfac_list[np] = 1
    CEWave_List[np] = ""
    speccorr_list[np] = ""
    //midc
    np += 1
    specname_list[np] = "midc"
    spec_list[np] = "midc"
    frag_list[np] = "frag_midc"
    IEfac_list[np] = 0.2
    //calfac_list[np]    //calculated automatically
    CEfac_list[np] = 1
    CEWave_List[np] = ""
    speccorr_list[np] = ""
    //full
    np += 1
    specname_list[np] = "fullc"
    spec_list[np] = "fullc"
    frag_list[np] = "frag_fullc"
    IEfac_list[np] = 0.2
    //calfac_list[np]    //calculated automatically
    CEfac_list[np] = 1
    CEWave_List[np] = ""
    speccorr_list[np] = ""
    //carbon
    np += 1
    specname_list[np] = "rBC"
    spec_list[np] = "rBC"
    frag_list[np] = "frag_lowc,frag_midc,frag_fullc"
    IEfac_list[np] = 0.2
    //calfac_list[np]    //calculated automatically
    CEfac_list[np] = 1
    CEWave_List[np] = ""
    speccorr_list[np] = ""

    //Place species into List for calculations:
    SVAR ms0_speclist = root:panel:ms0_speclist
    ms0_speclist = "Air,Water,NO3,SO4,Org,NH4,Chl,lowc,midc,fullc,rBC"

    setdatafolder root:
end



// 1.14F
//^ helper function for FD_Check_batch
// usage: FD_modifyBatch("m31", "m31", "frag_nitrate[31]", "31", "1", "1", "" )		// 1.44B fix
Function HR_FD_modifyBatch(spec_listStr, specname_listStr,frag_listStr, iefac_listStr, cefac_listStr, speccorr_listStr )
string spec_listStr, specname_listStr,frag_listStr, iefac_listStr, cefac_listStr, speccorr_listStr

	wave/t HR_spec_list = root:HR_frag:HR_spec_list
	wave/t HR_specName_list = root:HR_frag:HR_specName_list
	wave/t HR_specFamilyBase = root:HR_frag:HR_specFamilyBase
	wave/t HR_specFamilyExceptions = root:HR_frag:HR_specFamilyExceptions
	wave HR_specIEfac_list = root:HR_frag:HR_speciefac_list
	wave HR_specCEfac_list = root:HR_frag:HR_speccefac_list
	wave/T HR_CEWave_list = root:HR_frag:HR_CEWave_list
	wave/t HR_Speccorr_list = root:HR_frag:HR_speccorr_list
	wave/t HR_specFrag_list = root:HR_frag:HR_specfrag_list
	wave HR_specCalfac_list = root:HR_frag:HR_speccalfac_list

	wave HR_specColor_R = root:HR_frag:HR_specColor_R
	wave HR_specColor_G = root:HR_frag:HR_specColor_G
	wave HR_specColor_B = root:HR_frag:HR_specColor_B

	variable idex, num, flag=0

	num = numpnts(HR_spec_list)
	for (idex=0;idex<num;idex+=1)
		flag += stringmatch(spec_listStr,HR_spec_list[idex])
	endfor
	if (flag==0)
		Insertpoints num,1, HR_spec_list, HR_specName_list, HR_specFamilyBase, HR_specFamilyExceptions, HR_specFrag_list
		Insertpoints num,1, HR_specIEfac_list, HR_specCEfac_list, HR_CEWave_list, HR_speccorr_list, HR_specCalfac_list	
		Insertpoints num,1, HR_specColor_R, HR_specColor_G, HR_specColor_B	
		HR_spec_list[num] = spec_listStr
		HR_specName_list[num] = specname_listStr
		HR_specFrag_list[num] = frag_listStr
		HR_specIEfac_list[num] = Str2Num(iefac_listStr)
		HR_specCEfac_list[num] = Strlen(cefac_listStr)>=0? str2num(cefac_listStr) : HR_specCEfac_list[num-1]
		HR_Speccorr_list[num] = speccorr_listStr		
		HR_specFamilyExceptions[num] = ""		
		HR_CEWave_list[num] = ""	
//		HR_specCalfac_list[num] =1/Str2Num(iefac_listStr)  should be updated automatically
		HR_specColor_R[num] = 0		
		HR_specColor_G[num] = 0			// default to the blac color
		HR_specColor_B[num] = 0		
	endif	

End